/*
 * IBM Confidential
 *
 * IBM Rational Focal Point
 *
 * Copyright IBM Corporation 2010,2013. All Rights Reserved.
 */

// Input widget that addresses dynamic Bidi complex expressions
// (e.g. URIs, emails).
//
// @author bidi_bdl_lkemmel
//

//------------------------------------------------------------------------------
// BidiTextBox class and commonly used bidi utils
//------------------------------------------------------------------------------

//==============================================================================
// CONSTANTS
//==============================================================================

var gIsFF = navigator.userAgent.toLowerCase().indexOf("firefox") >= 0;
var gIsIE = !gIsFF && navigator.appName.toLowerCase().indexOf("microsoft") >= 0;

var BidiConstants = {};

// Direction constants
BidiConstants.LTR_DIR = 1;
BidiConstants.RTL_DIR = 2;
BidiConstants.CONTEXTUAL_DIR = 3;

BidiConstants.LRM = "\u200e";
BidiConstants.RLM = "\u200f";

BidiConstants.LRE = '\u202A';
BidiConstants.RLE = '\u202B';

BidiConstants.PDF = '\u202C';//pop directional format


// Regex for a left-to-right character
BidiConstants.LTR_CHAR = new RegExp("[a-zA-Z" + BidiConstants.LRM + "]");

// Keep these distinct, if we would need to support some language specific feature
// Regex for a Hebrew character
BidiConstants.HEBREW_CHAR = new RegExp("[\\u05be\\u05c0\\u05c3\\u05c6\\u05d0-\\u05f4"
        + BidiConstants.RLM + "\\ufb1d\\ufb1f-\\ufb28\\ufb2a-\\ufb4f]");

// Regex for an Arabic character
BidiConstants.ARABIC_CHAR = new RegExp("[\\u061b\\u0621-\\u064a\\u066d-\\u066f\\u0671-\\u06d5"
        + "\\u06dd\\u06e5\\u06e6\\u06ee\\u06ef\\u06fa-\\u070d\\u0710\\u0712-\\u072f"
        + "\\u074d-\\u07a5\\u07b1\\ufb50-\\ufd3d\\ufd50-\\ufdfc\\ufe70-\\ufefc]");

// Regex for control character (only address those we utilize)
BidiConstants.CONTROL_CHAR = new RegExp("[" + BidiConstants.LRM + BidiConstants.RLM
        + BidiConstants.LRE + BidiConstants.RLE + "]", "m");

// Delimiting strings to create Bidi segments
BidiConstants.LRM_STRING = BidiConstants.LRM + BidiConstants.LRM;
BidiConstants.RLM_STRING = BidiConstants.RLM + BidiConstants.RLM;

//==============================================================================
// BidiElemFactory namespace
//==============================================================================
var BidiElemFactory = {

//==============================================================================
// Extends the given element with the properties of BidiTextBox.
//==============================================================================
    subclass: function(id, direction, delimiters) {
        this.subclassElement(document.getElementById(id), direction, delimiters);
    },

    subclassElement: function(element, direction, delimiters) {
        if ((gIsIE || gIsFF) && element && "BidiTextBox" != element.subclass) {
            element = new BidiTextBox(element, direction, delimiters);
        }
    }
};

//==============================================================================
// BidiTextBox class
//==============================================================================

//==============================================================================
// BidiTextBox constructor
//==============================================================================
function BidiTextBox(element, direction, delimiters) {
    for (var property in BidiTextBox.prototype) {
        try {
            if (!element[property]) {
                element[property] = this[property];
            }
        } catch(e) {
            return element;
        }
    }
    element.delimiters = delimiters;

    if (direction == BidiConstants.CONTEXTUAL_DIR) {
        element.dir = "";
        element.isContextualDir = true;
        element.init();
    } else {
        if (direction == BidiConstants.RTL_DIR) {
            element.dir = "rtl";
        } else if (direction == BidiConstants.LTR_DIR) {
            element.dir = "ltr";
        }
        if (delimiters) {
            element.init();
            BidiTextBox.gBidiNodes.push(element);
        }
    }
    return element;
}

// Static members
BidiTextBox.gCurrentItem = null;
BidiTextBox.gUpdatedItem = null;
BidiTextBox.gInterval = null;
BidiTextBox.gBidiNodes = [];

BidiTextBox.prototype.subclass = "BidiTextBox";

//==============================================================================
// Init BidiTextBox object.
//==============================================================================
BidiTextBox.prototype.init = function() {
    this.connectListeners();

    var value = this.normalizeContent(false);
    this.formatContent(value, true);
};

if (gIsIE) {

//==============================================================================
// Connect event listeners for IE.
//==============================================================================
    BidiTextBox.prototype.connectListeners = function() {
        this.attachEvent("onfocus", this.onFocusIn);
        this.attachEvent("onkeyup", this.onKeyUp);
        this.attachEvent("oncut", this.onCut);
        this.attachEvent("onpaste", this.onPaste);
        this.attachEvent("ondrop", this.onPaste);
        window.attachEvent("onunload", handleUnload);
        if (this.delimiters) {
            this.attachEvent("onkeydown", this.onKeyDown);
            this.attachEvent("oncopy", this.onCopy);
            this.attachEvent("onmouseup", this.onMouseUp);
            this.attachEvent("oncontextmenu", this.onContextMenu);
            this.attachEvent("onsubmit", handleSubmit);
        }
    };

//==============================================================================
// Copy event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onCopy = function(event) {
        stopUpdate();
        BidiTextBox.gCurrentItem.resetSelection();

        var range = document.selection.createRange();
        if (range && range.text) {
            window.clipboardData.setData('Text', range.text.replace(/[\u200e\u200f]/g, ''));
            event = event || window.event;
            event.cancelBubble = true;
            event.returnValue = false;
            return false;
        }
    };

//==============================================================================
// Cut event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onCut = function(event) {
        BidiTextBox.gCurrentItem.onCopy(event);

        var range = document.selection.createRange();
        if (range) {
            BidiTextBox.gCurrentItem.hideText();
            range.text = "";
            setTimeout("BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };

//==============================================================================
// Paste event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onPaste = function(event) {
        stopUpdate();

        setTimeout("BidiTextBox.gCurrentItem.update(false);"
                + "BidiTextBox.gCurrentItem.getSelection();", 0);
    };

//==============================================================================
// Mouseup event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onMouseUp = function(event) {
        stopUpdate();
    };

//==============================================================================
// Contextmenu event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onContextMenu = function(event) {
        stopUpdate();

        BidiTextBox.gUpdatedItem = BidiTextBox.gCurrentItem;
        BidiTextBox.gInterval = setInterval(
                "BidiTextBox.gUpdatedItem.update(false)", 1000);
    };

//==============================================================================
// Keydown event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        stopUpdate();
        if (!event.ctrlKey && !event.altKey) {
            BidiTextBox.gCurrentItem.getSelection();
        }
    };
//==============================================================================
// Keyup event handler for IE.
//==============================================================================
    BidiTextBox.prototype.onKeyUp = function(event) {
        if (this.subclass != "BidiTextBox") {
            if(BidiTextBox.gCurrentItem == null || BidiTextBox.gCurrentItem == undefined) {
        	var inputs = document.getElementsByTagName("input");
        	for(var i=0; i<inputs.length; i++) {
        	  var input = inputs[i];
        	    if(input.name == "reportName") {
        	        BidiTextBox.gCurrentItem = input;
        	        break;
    		     }
        	}
            }
            if (!!BidiTextBox.gCurrentItem && !!BidiTextBox.gCurrentItem.onKeyUp) {
            return BidiTextBox.gCurrentItem.onKeyUp(event);
            } else {
                return null;
            }            
        }
        if (!event) {
            event = window.event;
        }
        if (!event.altKey) {
            if (!event.ctrlKey) {
                // 'Home', left arrow and right arrow key code values
                NAVIGATION_KEYS = [36, 37, 39];
                i = NAVIGATION_KEYS.length;
                while (i-- > 0) {
                    if (event.keyCode == NAVIGATION_KEYS[i]) {
                        this.getSafePosition();
                        return;
                    }
                }
            }
            this.update(false);
        }
    };

//==============================================================================
// Validates caret position after a key or mouse event occurs.
// The caret will not be placed between a Bidi formatting code and the
// associated character. To accomplish this, the caret may be moved 1 position
// forward or backwards, depending on navigation direction.
// MSIE-specific.
//==============================================================================
    BidiTextBox.prototype.getSafePosition = function() {
        if (!this.selection) {
            return;
        }
        var value = this.getValue();
        if (!value) {
            return;
        }
        var startMove, endMove;
        var prevSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (!this.selection) {
            return;
        }
        startMove = this.selection[0] - prevSelection[0];
        endMove = this.selection[1] - prevSelection[1];
    
        // First do the end. We may need to move both end and start (when selection
        // is collapsed), so have the start be updated also.
        if (startMove) {
            if (!this.isMovingOverControl(value, this.selection[0], startMove)) {
                startMove = 0;
            } else {
                startMove = startMove > 0 ? 1 : -1;
                do {
                    this.selection[0] = Math.max(0, this.selection[0] + startMove);
                } while (this.isMovingOverControl(value, this.selection[0], startMove));
                startMove = this.selection[0] - prevSelection[0];
            }
        }
        if (endMove && this.isMovingOverControl(value, this.selection[1], endMove)) {
                endMove = endMove > 0 ? 1 : -1;
            var textLen = value.length;
            do {
                this.selection[1] = Math.min(textLen, this.selection[1] + endMove);
            } while (this.isMovingOverControl(value, this.selection[1], endMove));
    
            if (this.selection[0] > this.selection[1]) {
                this.selection[0] = this.selection[1];
            }
            this.setSelection();
    
        } else if (startMove) {
            if (this.selection[0] > this.selection[1]) {
                this.selection[1] = this.selection[0];
                this.setSelection();
            } else {
                this.moveSelectionStart(startMove);
            }
        }
    };

//==============================================================================
// Checks if the caret is moving over a control character.
// MSIE-specific.
//==============================================================================
    BidiTextBox.prototype.isMovingOverControl = function(value, position, direction) {
        if (direction < 0) {
            return position - 1 >= 0
                && isBidiControlChar(value.charAt(position));
        }
        if (direction > 0) {
            return position + 1 <= this.getValue().length
                && isBidiControlChar(value.charAt(position - 1));
        }
        return false;
    };

//==============================================================================
// Moves selection start point, leaving the end intact.
// MSIE-specific.
//==============================================================================
    BidiTextBox.prototype.moveSelectionStart = function(value) {
        if (this.selection) {
            var range = document.selection.createRange();
            if (range) {
                range.moveStart('character', value);
                range.select();
            }
        }
    };

} else {

//==============================================================================
// Connect event listeners for Firefox.
//==============================================================================
    BidiTextBox.prototype.connectListeners = function() {
        this.addEventListener("focus", this.onFocusIn, false);
        this.addEventListener("change", this.update, true);
        this.addEventListener("input", this.update, true);
        this.addEventListener("drop", this.update, false);
        window.addEventListener("unload", handleUnload, true);
        if (this.delimiters) {
            this.addEventListener("keydown", this.onKeyDown, true);

            this.setAttribute("oncopy", ';');
            if (typeof this["oncopy"] == 'function') {
                this.removeAttribute("oncopy");
                this.addEventListener("copy", this.onCopy, true);
            }
            this.setAttribute("oncut", ';');
            if (typeof this["oncut"] == 'function') {
                this.removeAttribute("oncut");
                this.addEventListener("cut", this.onCopy, true);
            }
            window.addEventListener("submit", handleSubmit, false);
        }
    };

//==============================================================================
// Copy and cut event handler for Firefox.
//==============================================================================
    BidiTextBox.prototype.onCopy = function(event) {
        this.getSelection();
        if (this.selection && this.selectionStart != this.selectionEnd) {
            this.normalizeContent(true);

            setTimeout("BidiTextBox.gCurrentItem.formatContent(null, true);"
                    + "BidiTextBox.gCurrentItem.resetSelection();", 0);
        }
    };
    
//==============================================================================
// Keydown event handler for Firefox.
//==============================================================================
    BidiTextBox.prototype.onKeyDown = function(event) {
        if (!event) {
            event = window.event;
        }
        // older Firefox of version < 3
        if (event.ctrlKey && !event.shiftKey && !event.altKey
                && (event.keyCode == event.DOM_VK_C && typeof this["oncopy"] != 'function'
                || event.keyCode == event.DOM_VK_X && typeof this["oncut"] != 'function')) {
            this.onCopy(event);
        } else {
            this.getSelection();
        }
    };
}

//==============================================================================
// Sets global Bidi element to this object.
//==============================================================================
BidiTextBox.prototype.onFocusIn = function(event) {
    if (BidiTextBox.gCurrentItem == this
        || document.activeElement && BidiTextBox.gCurrentItem == document.activeElement) {
        return;
    }
    if (this.subclass != "BidiTextBox") {
        var target = event.srcElement || event.target;
        if (target && target.subclass == "BidiTextBox") {
            BidiTextBox.gCurrentItem = target;
        }
    } else {
        BidiTextBox.gCurrentItem = this;
    }
};

//==============================================================================
// Inserts controls characters after delimiters and adjusts locally saved
// selection.
//==============================================================================
BidiTextBox.prototype.formatContent = function(value, applyValue) {
    if (!value) {
        value = this.getValue();
    }
    if (this.isContextualDir) {
        this.dir = resolveTextDirection(0, value);
    }
    if (!this.delimiters) {
        return value;
    }
    if (this.dir == "ltr" && !isBidiText(value)
        || this.dir == "rtl" && !hasLtrText(value)) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var CONTROL_STR = this.dir == "rtl" ? BidiConstants.RLM_STRING
        : BidiConstants.LRM_STRING;
    CONTROL_STR_LEN = CONTROL_STR.length;

    var newStr = new String(value);
    var i, j, c;
    var start = 0, end = -1;
    var afterStrong = false;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length - 1; i++, j++) {
        c = value.charAt(i);
        if (isBidiText(c)) {
            afterStrong = this.dir == "ltr";
        } else if (c.match(BidiConstants.LTR_CHAR)) {
            afterStrong = this.dir == "rtl";
        } else if (afterStrong && this.delimiters.indexOf(c) >= 0) {
            newStr = newStr.substr(0, j + 1) + CONTROL_STR + newStr.substr(j + 1);
            afterStrong = false;
            if (this.selection) {
                if (this.selection[0] == i) {
                    ++start;
                } else if (this.selection[0] > i) {
                    start += CONTROL_STR_LEN;
                }
                if (this.selection[1] == i) {
                    ++end;
                } else if (this.selection[1] > i) {
                    end += CONTROL_STR_LEN;
                }
            }
            j += CONTROL_STR_LEN;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.setValue(newStr, true);
    }    
    this.unhideText();
    return newStr;
};

//==============================================================================
// Gets and caches the current native selection.
// Returns integer array containing the selection start and end points.
//==============================================================================
BidiTextBox.prototype.getSelection = function() {
    if (typeof this.selectionStart == "number") {
        this.selection = [this.selectionStart, this.selectionEnd];
    } else if (document.selection) {
        try {
            var start = 0;
            var range = document.selection.createRange().duplicate();
            var range2 = range.duplicate();
            var rangeLength = range.text.length;
            
            if (this.isMultiLine) {
                range2.moveToElementText(this);
            } else {
                range2.expand('textedit');
            }
            // Return value of range#moveStart doesn't seem reliable, so
            // also limit number of iterations.
            // Otherwise we can fall into infinite loop.
            var n = range2.text.length;
            while (start < n && range.compareEndPoints('StartToStart', range2) > 0) {
                if (range.text.charAt(0).match(/[\r\n]/)) {
                    if (!range.moveStart("word", -1)) {
                        break;
                    }
                } else if (!range.moveStart('character', -1)) {
                    break;
                }
                ++start;
            }
            this.selection = [start, start + rangeLength];
        } catch (e) {
            this.selection = null;
        }
    }
};

//==============================================================================
// Gets text value.
//==============================================================================
BidiTextBox.prototype.getValue = function() {
    return this.value || this.text || "";
};

//==============================================================================
// Form submit event handler.
//==============================================================================
BidiTextBox.prototype.handleSubmit = function(event) {
    if (gIsIE) {
        stopUpdate();
    }
    this.resetSelection();
    this.normalizeContent(true);
};

//==============================================================================
// Sets text color to the background color making that way the text invisible.
//==============================================================================
BidiTextBox.prototype.hideText = function() {
    if (typeof this.foreground == 'undefined') {
        this.foreground = this.style.color;
    }
    this.style.color = this.style.backgroundColor || "white";
};

//==============================================================================
// Removes controls characters and adjusts locally saved selection.
//==============================================================================
BidiTextBox.prototype.normalizeContent = function(applyValue) {
    var value = this.getValue();
    if (!value || !this.delimiters) {
        return value;
    }
    var length = value.length;
    if (length < 1) {
        return value;
    }
    var newStr = new String(value);
    var i, j;
    var start = 0, end = -1;

    if (this.selection) {
        start = this.selection[0];
        end = this.selection[1];
    }
    for (i = 0, j = 0; i < length; i++) {
        if (isBidiControlChar(value.charAt(i))) {
            newStr = newStr.substr(0, j) + newStr.substr(j + 1);
            if (this.selection) {
                if (this.selection[0] > i) {
                    --start;
                }
                if (this.selection[1] > i) {
                    --end;
                }
            }
        } else {
            ++j;
        }
    }
    if (this.selection) {
        this.selection = [Math.min(start, end), end];
    }
    if (applyValue && newStr != value) {
        this.hideText();
        this.setValue(newStr, true);
    }
    return newStr;
};

//==============================================================================
// Resets our private selection object.
//==============================================================================
BidiTextBox.prototype.resetSelection = function() {
    this.selection = null;
};

//==============================================================================
// Sets the native selection.
//==============================================================================
BidiTextBox.prototype.setSelection = function() {
    if (!this.selection || this.readOnly) {
        return;
    }
    if (this.setSelectionRange) {
        if (this.selectionStart != this.selection[0] || this.selectionEnd != this.selection[1]) {
            this.focus();
            this.setSelectionRange(this.selection[0], this.selection[1]);
        }
    } else if (gIsIE) {
        var newSelection = [this.selection[0], this.selection[1]];
        this.getSelection();
        if (newSelection[0] != this.selection[0] || newSelection[1] != this.selection[1]) {
            var range = this.createTextRange() || document.body.createTextRange();
            if (range) {
                if (this.isMultiLine) {
                    range.moveToElementText(item);
                } else {
                    range.expand('textedit');
                }
                range.collapse(true);
                range.moveEnd('character', newSelection[1]);
                range.moveStart('character', newSelection[0]);
                range.select();
            }
        }
    }
};

//==============================================================================
// Sets the text value.
//==============================================================================
BidiTextBox.prototype.setValue = function(newValue, setSelection) {
    if (!newValue) {
        newValue = this.newValue;
    }
    if (newValue != this.getValue()) {
        if (this.text) {
            this.text = newValue;
        } else if (this.value) {
            this.value = newValue;
        }
        if (setSelection) {
            this.setSelection();
        }
    }
};

//==============================================================================
// Resets element's text color.
//==============================================================================
BidiTextBox.prototype.unhideText = function() {
    this.style.color = this.foreground;
};

//==============================================================================
// Updates the text value though normalizing and subsequent formatting.
//==============================================================================
BidiTextBox.prototype.update = function(delay) {
    if (gIsFF) {
        // On load FF fires focus on body element, and as a result
        // BidiTextBox.gCurrentItem can be uninitialized.
        BidiTextBox.gCurrentItem = this;
    }
    this.getSelection();

    var origValue = this.getValue();
    this.newValue = this.formatContent(this.normalizeContent(false), false);

    if (this.newValue != origValue) {
        this.hideText();
        if (delay == true) {
            setTimeout("BidiTextBox.gCurrentItem.setValue(null, true);"
                    + "BidiTextBox.gCurrentItem.unhideText();"
                    + "BidiTextBox.gCurrentItem.resetSelection();", 0);
        } else {
            this.setValue(this.newValue, true);
            this.unhideText();
            this.resetSelection();
        }
    }
};

//==============================================================================
// Handle submit events in this window.
//==============================================================================
function handleSubmit(event) {
    for (var i = BidiTextBox.gBidiNodes.length; i-- > 0; ) {
        BidiTextBox.gBidiNodes[i].handleSubmit(event);
    }
}

//==============================================================================
// Handle window unload.
//==============================================================================
function handleUnload(event) {
    BidiTextBox.gBidiNodes = [];
    BidiTextBox.gCurrentItem = null;
}

//==============================================================================
// Stops item update invocation.
//==============================================================================
function stopUpdate() {
    if (BidiTextBox.gInterval) {
        clearInterval(BidiTextBox.gInterval);
        BidiTextBox.gInterval = null;
    }
    BidiTextBox.gUpdatedItem = null;
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================
function isBidiText(text) {
    return BidiConstants.HEBREW_CHAR.test(text)
            || BidiConstants.ARABIC_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains LTR characters.
//==============================================================================
function hasLtrText(text) {
    return BidiConstants.LTR_CHAR.test(text);
}

//==============================================================================
// Checks whether or not the given string contains Bidi characters.
//==============================================================================

function isBidiControlChar(c) {
    return c.match(BidiConstants.CONTROL_CHAR);
}

//==============================================================================
// Returns direction resolved to LTR or RTL.
//==============================================================================
function resolveTextDirection(dir, value) {
    if (dir && dir != "contextual" && dir != "default") {
        return dir;
    }
    if (value) {
        for (var i = 0, n = value.length; i < n; i++) {
            var c = value.charAt(i);
            if (isBidiText(c)) {
                return "rtl";
            }
            if (hasLtrText(c)) {
                return "ltr";
            }
        }
    }
    return getBidiDir(2);
}

//==============================================================================
// Extends text controls with Bidi properties and applies base text direction
// for both dynamic and static elements.
//==============================================================================
var DYN_TAG_NAMES = ['textarea'];

function handleBidi() {
    if (typeof getBidiDir == 'function') {
        var nodes = document.getElementsByTagName('input');
        for (var i = nodes.length; i-- > 0;) {
            if (/^text/.test(nodes[i].type)) {
                if (/bidi_url/.test(nodes[i].className)) {
                    if (typeof BidiElemFactory == 'object') {
                        BidiElemFactory.subclassElement(nodes[i],
                            BidiConstants.LTR_DIR, ".:@\\/");
                    }
                } else {
                    setDynamicElementDirection(nodes[i]);
                }
            }
        }
        //Don't put ltr on select tag if lang dir or contextual is rtl or
        //the dropdown button will not be rtl mirrored.
        //Note: This does not allow select to implement BTD if ltr
        nodes = document.getElementsByTagName('select');
        for (var i = nodes.length; i-- > 0;) {
            var dir = resolveTextDirection(getBidiDir(0), nodes[i].nodeValue);
            if (getBidiDir(2) == 'rtl' && dir == 'rtl') {
                setDynamicElementDirection(nodes[i].nodeValue);
            }
        }
        nodes = document.getElementsByTagName('option');
        for (var i = nodes.length; i-- > 0;) {
            var dir = resolveTextDirection(getBidiDir(0), nodes[i].innerHTML);
            var controlChar = dir == 'rtl' ? BidiConstants.RLE : BidiConstants.LRE;
            nodes[i].innerHTML = controlChar + nodes[i].innerHTML + BidiConstants.PDF;
            controlChar = "";
            if (dir == 'ltr') {
                controlChar = getBidiDir(2) == 'ltr' ? BidiConstants.LRM : "";
            } else {
                controlChar = getBidiDir(2) == 'rtl' ? BidiConstants.RLM : "";
            }
            if (controlChar != "") {      
                //fixParentheses
                nodes[i].innerHTML = nodes[i].innerHTML.replace(/\(/g, controlChar + '('); 
            }
        }
       for (var i = DYN_TAG_NAMES.length; i-- > 0;) {
            var nodes = document.getElementsByTagName(DYN_TAG_NAMES[i]);
            for (var j = nodes.length; j-- > 0;) {
                setDynamicElementDirection(nodes[j]);
            }
        }
        nodes = getStaticBidiElements();//nodes with class='bidiText'
        for (var i = nodes.length; i-- > 0;) {
            setStaticElementDirection(nodes[i]);
        }
    }
}
//==============================================================================
// Applies base text direction for dynamic element.
//==============================================================================
function setDynamicElementDirection(node) {
    if (node && !node.dir) {
        bidiTextDir = getBidiDir(0);
        if (bidiTextDir != "contextual" && bidiTextDir != "default") {            
            node.dir = bidiTextDir;
        } else if (typeof BidiElemFactory == 'object') {
            BidiElemFactory.subclassElement(node,
                BidiConstants.CONTEXTUAL_DIR, "");
        }
    }
}

//==============================================================================
// Applies base text direction for static element.
//==============================================================================
function setStaticElementDirection(node) {
    if (!node || /bidi_url/.test(node.className)) {
        return;
    }
    if (node.nodeType == '3' && /[^\s]+/.test(node.nodeValue)) {
        var dir = resolveTextDirection(getBidiDir(0), node.nodeValue);
        if (dir == 'ltr') {
            if (getBidiDir(2) == 'rtl') {
                setTextNodeDirection(node, "ltrRightAlign");
            } else {
            	fixParentheses(node, BidiConstants.LRM);
            }
        } else {
        	if (getBidiDir(2) == 'ltr') {
        		setTextNodeDirection(node, "rtlLeftAlign");
        	} else {
            	fixParentheses(node, BidiConstants.RLM);
            }
        }
    } else {
        for (var i = node.childNodes.length; i-- > 0;) {
            setStaticElementDirection(node.childNodes[i]);
        }
    }
}

// Code added to handle the case of English then Arabic then ( in LTR
// Or Arabic then English then ( in RTL
function fixParentheses(node, markup) {
	node.nodeValue = node.nodeValue.replace(/\(/g, markup + '('); 
}

//==============================================================================
// Applies base text direction for a text node.
//==============================================================================
function setTextNodeDirection(node, className) {
    if (node) {
        if (node.nodeValue) {
            node.nodeValue = node.nodeValue.replace(/[\u00a0]$/gm, "");
        }
        node = node.parentNode;
        if (node && node.tagName && node.tagName.toUpperCase() != 'SCRIPT') {
            // When there is no block-level element parented to this node,
            // use immediate parent anyway
            var parent = node;
            var found = false;
            while (node && resolveStyle(node, 'display') == 'inline') {
                node = node.parentNode; 
                found = true;                
            }
            if (found && node) {
                parent = node;
            }            
            if (!/ltrRightAlign|rtlLeftAlign/.test(parent.className)) {
                parent.className += (parent.className ? " " : "") + className;
            }
        }
    }
}

function resolveStyle(node, style) {
    var gcs, ret;
    try {
        gcs = "getComputedStyle" in window;
        ret = (gcs ? window.getComputedStyle(node, null) : node.currentStyle).getPropertyValue(style);         
    } catch ( err ) {
    }
    return ret;
}


var CLASS_NAMES = {
	bidiText: 'bidiText'
 };
var TAG_NAMES = ['TR', 'TD', 'a', 'h1'];

//==============================================================================
// Retreives document static elements enabled for bidi.
//==============================================================================
function getStaticBidiElements() {
    var bidiNodes = [];
    for (var i = TAG_NAMES.length; i-- > 0;) {
        var nodes = document.getElementsByTagName(TAG_NAMES[i]);
        for (var j = nodes.length; j-- > 0;) {
            var classNames = nodes[j].className.split(' ');
            for (var k = classNames.length; k-- > 0;) {
                if (CLASS_NAMES[classNames[k]]) {
                    bidiNodes.push(nodes[j]);
                    break;
                }
            }
        }
    }
    return bidiNodes;
}

//Apply unicode characters based on user preferences Base Text Direction//
function applyUCC(text) {
    var uccText = text;
    if (uccText != "") {
        var dir = resolveTextDirection(getBidiDir(0), uccText);
        var controlChar = dir == "ltr" ? BidiConstants.LRE : BidiConstants.RLE;
        uccText = controlChar + uccText + BidiConstants.PDF;
    }
    return uccText;
}

//Apply unicode characters based on language directionality//
//This is used with messages in javascript alerts.//
function applyDirection(text) {
    var uccText = text;
    if (uccText != "") {
        var dir = getBidiDir(2);
        var controlChar = dir == "ltr" ? BidiConstants.LRE : BidiConstants.RLE;
        uccText = controlChar + uccText + BidiConstants.PDF;
    }
    return uccText;
}
