// BEGIN COPYRIGHT
//*************************************************************************
//
// Licensed Materials - Property of IBM
// 5725-A09
// Copyright IBM Corporation 2010. All Rights Reserved.
// US Government Users Restricted Rights- Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
//
//*************************************************************************
//END COPYRIGHT

// Wrapper class around the DatePicker to have to entry fields: one for date, one for time

/*global dojo,com,dijit,console */

dojo.provide("com.lombardisw.DateTimePicker");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");

dojo.declare("com.lombardisw.DateTimePicker", [dijit._Widget, dijit._Templated], { 
    templateString: '' +
    '<div dojoAttachPoint="pickerDiv" id="widget_${id}">' +
    '  <span dojoAttachPoint="leftPicker"></span>' +
    '  <span dojoAttachPoint="rightPicker"></span>' +
    '  <input dojoAttachPoint="hiddenInput" style="display:none" id=${id} name=${name} type="text" isDateField="true" />' +
    '</div>',
    
    statics: {
        datePattern: "yyyy-MM-dd",
        timePattern: "HH:mm:ss"
	},

    constructor: function(args){
    
        var options = {};
        if (args.value) {
            options.value = args.value;
        }

        if (args.datePackage) {
        	options.datePackage = args.datePackage;
        }
        
				// Note: do not pass args to child widgets, else we ge dup ids, and the childs can't register with dijit
        this.dateBoxWidget = new dijit.form.DateTextBox(options);
		//In dojo 1.4.3, constructor of _DateTimeTextBox needs argument, so we add options as parameter 
        this.timeBoxWidget = new dijit.form.TimeTextBox({}); 
        this.timeBoxWidget.value = this.dateBoxWidget.value;
        
    },
    
    postCreate: function(){
        this.dateBoxWidget.placeAt(this.leftPicker);
        this.timeBoxWidget.placeAt(this.rightPicker);
				this.timeBoxWidget.startup();
        
        this.connect(this.timeBoxWidget, "onChange", "syncDateToInputField");
        this.connect(this.dateBoxWidget, "onChange", "syncDateToInputField");
    },
    
    // overwrite the attr function when someone whats to retrive the value
    attr: function attr(name, value){
        var rc;
        if (name == "value") {
            if (value == undefined) {
                // someone wants to read value. Orig. attr returns only date
                this.dateBoxWidget.value.setHours(this.timeBoxWidget.value.getHours());
                this.dateBoxWidget.value.setMinutes(this.timeBoxWidget.value.getMinutes());
                this.dateBoxWidget.value.setSeconds(this.timeBoxWidget.value.getSeconds());
                rc = this.dateBoxWidget.value;
            }
            else {
                // set the date and time
                this.timeBoxWidget.attr("value", value); // time part
                rc = this.dateBoxWidget.attr("value", value); // date part
            }
        }
        return rc;
    },
    
    // also overwrite depracted setter/getters
    setValue: function(value){
        this.attr("value", value);
    },
    getValue: function(){
        return this.attr("value");
    },
    
    // sync the date picker to the hidden input
    syncDateToInputField: function(){
        var dateFormat = {
            datePattern: this.statics.datePattern,
            selector: "date"
        };
        var timeFormat = {
            timePattern: this.statics.timePattern,
            selector: "time"
        };
        
        // default time settings: current day, 0:00 
        if (!this.dateBoxWidget.value) {
            this.dateBoxWidget.value = new Date();
        }
        if (!this.timeBoxWidget.value) {
            this.timeBoxWidget.value = new Date(0, 0, 0, 0, 0, 0, 0);
        }
        if (this.dateBoxWidget.value.toGregorian) {
        	this.hiddenInput.value=this.dateBoxWidget.format(this.dateBoxWidget.value.toGregorian(), dateFormat) + " " + this.timeBoxWidget.format(this.timeBoxWidget.value, timeFormat);
        } else {
         	this.hiddenInput.value=this.dateBoxWidget.format(this.dateBoxWidget.value, dateFormat) + " " + this.timeBoxWidget.format(this.timeBoxWidget.value, timeFormat);
        }
    }
});
