/*
Licensed Materials - Property of IBM
5725-A09
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

dojo.provide("com.lombardisw.Dialog");
dojo.require("dijit.Dialog");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

/** Ported from Business Space */
dojo.declare("com.lombardisw.Dialog", [dijit._Widget], {
	popup : null,
	rtl: "",    

	constructor: function(parameters){
		//Fixes IPAD defect, where if draggable is set to true (X) button does not work on IPAD.
		//Still allows for manual override though
		if(typeof parameters == "undefined") parameters = {"draggable" : false};
		if(typeof parameters.draggable == "undefined") parameters.draggable = false;
		
		dojo.mixin(this, parameters);

		// Save the parameter object (minus the templateString) to pass to the dijit.Dialog so title, style, etc will all be provided.
		this.dialogParams = parameters;
		if (this.dialogParams && this.dialogParams.templateString) delete this.dialogParams.templateString;
	},
	
	// * set variables for the template
	postMixInProperties: function() {
		if (this.dialogParams) {
		this.popup = new dijit.Dialog(this.dialogParams);
		} else {
			this.popup = new dijit.Dialog({});
		}
		this.inherited(arguments);
	},
	/*
	 * Called after the template is loaded, and wraps any controls with the common UI styling if possible.
	 * Note that if you override this method and programatically add any controls, you will want to call
	 * this.inherited(arguements) after the logic of your method so that those controls get automatically
	 * wrapped as well.
	 */
	postCreate: function() {
        if (this.popup) {        
    		if (!this.title || this.title == "") {
    			this.popup.titleBar.style.borderBottom = "0px";
    		}
            if (this.message && this.message.id) {
                dojo.attr(this.popup.domNode,"aria-describedby",this.message.id);
            }
    		if (this.popup.closeButtonNode) {
                if (this.dialogParams && this.dialogParams.cancelLabel) {            
                    dojo.attr(this.popup.closeButtonNode, "title", this.dialogpParams.cancelLabel);		
                }
            }
        }
	},
	/*
	 * The dialog is centered on the screen based on it's size, so if you want to override the position (for
	 * example to create a full screen dialog that has position top 0, left 0), you will want to override the
	 * show method and manipulate the sytle member of this.popup.domNode.
	 */ 
	show: function(){
		if (!this.loaded) {
			this.popup.attr("content", this.domNode);
			this.loaded = true;
		}
		
		if(this.isIPad) {
			this.positionForIPad();
		}
		
		this.popup.show();
	},
	hide: function(){
		this.popup.hide();
	},	
	/*
	 * Adds a button to the dialog.  This assumes that the designer has provided an element in the template
	 * with a dojoAttachPoint of "buttonContainer".  Use the class name "bspace-dialog-buttonContainer" to gain common styling.
	 *
	 * id - the ID that will be assigned to the button element
	 * value - the text that will be displayed in the button element
	 * callback - a callback function that will be hitched to the onclick event of the button
	 * context - the context of the callback function
	 */
	addButton: function(id, value, callback, context) {
		if (this.buttonContainer) {
			var button = document.createElement("button");
			button.innerHTML = value;
            dojo.addClass(button,"dijitReset dijitInline dijitButtonText");
			button.id = id;

			// If context is not provided or undefined, use a null context
			if (context==undefined) {
				dojo.attr(button, "onclick", dojo.hitch(null, callback));
			} else {
				dojo.attr(button, "onclick", dojo.hitch(context, callback));
			}
			this.buttonContainer.appendChild(button);
		}
	},
	
	isIPad: navigator.userAgent.indexOf("iPad") > -1,

	positionForIPad: function() {
		// dijit.Dialog._position() is called twice at the beginning, only position the dialog that time, later let it there,
		// when rotating iPad always finds a good position to keep the dialog visible
		// when the keypad is expanded the dialog the focused input field does not get anymore hidden this way   
		this.popup._positionCount = 0;
		this.popup.__position = this.popup._position;
		this.popup._position = dojo.hitch(this.popup, function(){
		  // _position needs to be called twice during initialization
		  if (this._positionCount < 2) {
			this._positionCount++;
			this.__position();
			this.domNode.style.top = (30 + window.scrollY) + "px";
		  }
		});
	},
	
	destroy: function(finalize) {
		if (this.popup) {
			this.popup.destroy();
			this.popup = null;
		}
	    this.inherited('destroy', arguments);
	}
	
});

