/*
Licensed Materials - Property of IBM
5725-A09
(C) Copyright IBM Corporation 2013. All Rights Reserved.
US Government Users Restricted Rights- Use, duplication or disclosure
restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 * Ported from bpmSocial ErrorDialog
 */
dojo.provide("com.lombardisw.InfoDialog");

dojo.require("com.lombardisw.Dialog");

dojo.declare("com.lombardisw.InfoDialog", [com.lombardisw.Dialog, dijit._Templated], {

  templateString: "<div><div class='portal-support-dialog-box'>"
    + "<div dojoAttachPoint='message' class='portal-support-dialog-message' id='message_${id}'></div>"
    + "<div dojoAttachPoint='detailsTitle' style='display:inline'></div>"
    + "<div dojoAttachPoint='detailsContent' class='portal-support-dialog-details-content'></div>" + "</div>"
    + "<div dojoAttachPoint='buttonContainer' class='dijitReset dijitInline dijitButtonNode'></div>"
    + "<div style='clear: both'></div>" + "</div>",

  _message: "",
  _label: null,
  _details: null,

  /**
   * Creates a popup which will show a message.
   * 
   * @param message The message to be shown
   * @param details The details to be shown
   * @param label The label of the confirmation button
   * 
   * Please use the static method below instead of directly instanciating the popups.
   * 
   */
  constructor: function(params){
    if (typeof params.message === 'string') {
      this._message = params.message;
    }
    if (typeof params.label === 'string') {
      this._label = params.label;
    }
    if (typeof params.details === 'string') {
      this._details = params.details;
    }
    
  },

  postCreate: function postCreate_InfoDialog(){
    this.message.innerHTML = this._message;
    var self = this;
    if (this._label != null) {
      this.addButton(this.id + "_confirmationButton", this._label, function(){
        self.hide();
      });
    }

    // If details exist, create a title pane for it.
    if (this._details) {

      var detailsTitlePic = document.createElement("div");
      detailsTitlePic.innerHTML = "+";
      dojo.addClass(detailsTitlePic, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitlePic);

      var detailsTitleLabel = document.createElement("div");
      detailsTitleLabel.innerHTML = "dialog_details_more";
      dojo.addClass(detailsTitleLabel, "bpm-social-dialog-details-title");
      this.detailsTitle.appendChild(detailsTitleLabel);

      this.detailsTitle.tabIndex = "0";

      dojo.style(this.detailsContent, "display", "none");
      this.detailsContent.innerHTML = this._details;

      var detailsHidden = true;
      this.connect(this.detailsTitle, "ondijitclick", function(e){
        if (detailsHidden) {
          dojo.style(this.detailsContent, "display", "block");
          detailsHidden = false;
          detailsTitlePic.innerHTML = "-";
          detailsTitleLabel.innerHTML = "dialog_details_hide";
        }
        else {
          dojo.style(this.detailsContent, "display", "none");
          detailsHidden = true;
          detailsTitlePic.innerHTML = "+";
          detailsTitleLabel.innerHTML = "dialog_details_more";
        }
      });
    }
    else {
      dojo.style(this.detailsTitle, "display", "none");
      dojo.style(this.detailsContent, "display", "none");
    }

    this.connect(this.popup, 'onHide', "_finish");
    this.inherited("postCreate", arguments);
  },

  _finish: function _finish_InfoDialog(){
    // Trigger any subsequent user actions 
    // Synchronous call to event handler methods.
    this.onClose();
    // Basically this is also a dijit._Widget with a meaningful destroy method.#
    // Because the onHide event is triggered within a fade, one cannot immediately destroy everything.
    setTimeout(dojo.hitch(this, this.destroy), 1000);
  },

  /**
   * Event handling hook.
   */
  onClose: function onClose_InfoDialog(){

  }

});

/**
 * Shows an info popup. For example in case an ad hoc event has been started but no immediate task is
 * available.
 * 
 * I.e. dojo.require("com.lombardisw.InfoDialog");
 * com.lombardisw.InfoDialog.showInfo(message, label);
 * 
 * @param message The message to be shown
 * @param details The details to be shown

 */
com.lombardisw.InfoDialog.showInfo =
  function(message /* string */, label /* string */){
    var dialog = new com.lombardisw.InfoDialog({
      "label": label,
      "message": message,
      "class": "portal-support-dialog-info"
    });
    dialog.show();
  };

