/**********************************************************************
* Licensed Materials - Property of IBM
*
* 5724-R31, 5655-S30
*
* (C) Copyright IBM Corp. 2011,2013. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
**********************************************************************/

/**
 * Check if `TW` namespace is available.
 */
if (typeof TW == 'undefined') {
    var TW = {};
    TW.pageData = {};
};

/**
 * Initializes page resources;
 */
(function() {
    
    TW.pageData.servletURL = "criticalPath";
    
    /**
     * Date format used by calendar.
     */
    TW.pageData.dateFormat = getDateFormatPattern();
	TW.pageData.timeFormat = getTimeFormatPattern();

    if (getCalendarType() == "islamic") {
        TW.pageData.dateFormat = "MMMM d, yyyy h:mm:ss a";//must match BidiUtils.java
        TW.pageData.timeFormat = "h:mm:ss a";
    } else if (getCalendarType() == "hebrew") {
        TW.pageData.dateFormat = "dd MMMM yyyy HH:mm:ss";//must match BidiUtils.java
        TW.pageData.timeFormat = "HH:mm:ss";
    }

    /**
     * Page text resources.
     */
    TW.message = {
        changesWillBeReset: changeWillBeReset_MsgI18n,
        showAllActivities: showAll_MsgI18n,
        hideNonCriticalActivities: hideActivities_MsgI18n,
        serverTimeout: timeout_MsgI18n,
        serverError: dataNotAvailable_MsgI18n,
        actionFailed: actionFail_MsgI18n,
        valueModified: valueModified,
        overdue: overdue,
        enterNumber: enterNumber,
        calendar: calendar,
        dueDate: dueDate,
        empty: "<empty>"        
    };

    /**
     * Message to be shown if page is blocked.
     */
    //dojo.require("dijit.Dialog");
    //TW.pageData.loadingDialog = new dijit.Dialog({style: "width: 300px"}); 
    //TW.pageData.loadingDialog.setContent("<div class='blockMsg'><br/><img id='blockUIimage' src='images/ajax-loader.gif' width='16px' height='16px' >&nbsp;<span>Refreshing information...</span><br/><br/></div>");  
    //dojo.query(".dijitDialogTitleBar").style("display", "none");
    //TW.pageData.loadingDialog._onKey = function(){};
    //dojo.query("body").addClass("soria");
    TW.pageData.loadingDialog = {
    		show: function(){ },
    		hide: function(){ }
    };
    //

    /**
     * Store old toolbar handler.
     */
    TW.oldPerformNonPopupAction = performNonPopupAction;
    
    /**
     * Wrap toolbar calls into async requests. 
     */
    performNonPopupAction = function(prefix, url, idString) {
        url = prefix + '/' + url;
            if (url.indexOf('?') > -1) {
            url += '&';
        } else {
            url += '?';
        }
        url += idString;
        
        jQuery.ajax({
            type: "GET",
            url: url,
            success: function() {onSelectionChange();},
            failure: function() {alert(TW.message.actionFailed);},
            dataType: "html"
        });
    };

})();

/**
 * Executed onLoad().
 */
$(document).ready(function() {
    /**
    * The "standard" English duration units that are 
    * being used by the legacy code
    */
    TW.englishDurationUnits = [
        "day",
        "days",
        "minute",
        "minutes",
        "month",
        "months",
        "hour",
        "hours"
    ];

    /**
    * Contains translated strings of duration units
    * mapping English -> Target Language
    */
    TW.translatedDurationUnits = {
        hour: hourSingular,
        hours: hourPlural,
        day: daySingular,
        days: dayPlural,
        minute: minuteSingular,
        minutes: minutePlural,
        month: monthSingular,
        months: monthPlural
    };

    /**
    * Contains the English duration unit equivalents of the 
    * translated duration units
    */
    TW.translatedToEnglishDurationUnits = {};
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["hour"]] = "hour";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["hours"]] = "hours";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["day"]] = "day";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["days"]] = "days";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["minute"]] = "minute";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["minutes"]] = "minutes";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["month"]] = "month";
    TW.translatedToEnglishDurationUnits[TW.translatedDurationUnits["months"]] = "months";

    var node = document.getElementById("body");
    node.focus();

    if ($("body").hasClass("dijit_a11y")) {
        $("#showCurrentArrow").attr("src","images/arrow_left_white.png");
        $("#hideCurrentArrow").attr("src","images/arrow_right_white.png");
    }
    
    TW.checkIfDebugModeAndInit();
    
    $("select#path").val(TW.pageData.cpType);

    if (TW.pageData.cpType == 3) {
        $("select#pathHistoricalScenario").css("display", "inline").val(TW.pageData.hasId);
		}

    var params = {action: "GET"};

    if (TW.pageData.printable) {
        params.printable = true;
    }
    
    TW.sendAjaxRequest(params, TW.populateInstanceData);
    
    TW.initHandlersOnPageLoad();
});

/**
 * Performs async server request.
 *  
 * @param params - JSON parameters will be passed to server.
 * @param callback - function will be called back after request finished and server returned answer.
 */
TW.sendAjaxRequest = function(params, callback) {

    assert(params && params.action && callback, "Action and callback must be specified to perform AJAX call.");

    params.bpdInstanceId = TW.pageData.bpdInstanceId;
    params.cpType = TW.pageData.cpType;
    params.hasId = TW.pageData.hasId;

    TW.pageData.loadingDialog.show();
    jQuery.ajax({
			type: "POST",
			url: TW.pageData.servletURL,
			data: params,
			success: callback,
            error: TW.processAjaxError,
			dataType: "json"
    });
};

/**
 * Called back if AJAX request timed out or failed.
 * 
 * @see jQuery documentation for parameter values.
 */
TW.processAjaxError = function(XMLHttpRequest, textStatus, errorThrown) {
    TW.pageData.loadingDialog.hide();
    switch(textStatus) {
        case "timeout":
            if (confirm(TW.message.serverTimeout)) {
                TW.pageData.loadingDialog.show();
                jQuery.ajax(this);
            }
            break;
        default:
            if (confirm(TW.message.serverError)) {
                window.location.reload();
            }
    };
};

/**
 * Loads instance data on first page load.
 * @param instanceData JSON with page data.
 */
TW.populateInstanceData = function(instanceData) {

    // Populate instance details
    TW.pageData.bpdId = instanceData.values.bpdId;
    TW.pageData.bpdName = instanceData.values.bpdName;

    for(var prop in instanceData.values) {
        $("#" + prop).html(instanceData.values[prop]);
    }

    $("#projectedDueDate").addClass(instanceData['analysis']);
    $("#dueDate").addClass(instanceData['dueDateAnalysis']);
    $("#projectedDueDate").attr("tw:default", $("#projectedDueDate").text());
    
    $("#updateDueDate").defVal(instanceData.values.dueDate);
    
    // Load priority list into Mass Updates.
    $(".massUpdatePriorities").html(instanceData.priorities);
    
    // Load CP table.
    $("tbody#dueDatesContent").html(instanceData.table);

    // What to hide to display Printable view.
    if (TW.pageData.printable) {
        $("div#header").hide();
        $("td#nav").hide();
        $("td#nav-bottom").hide();
        $("p#helpText").hide();
        $("div#toolbar").hide();
        $("table#massUpdates2").hide();
        $("div#changesContainer").hide();
        $(".completed").hide();
        $("#printable").hide();

        $(".hideInPrintable").hide();
        $("#showCurrentArrow").click();
        
        $("#currentDueDateTitle").text("Projection:");
        var projection = $("#path option:selected").text();
        if (TW.pageData.cpType == 3) {
            projection += " - " + $("select#pathHistoricalScenario option:selected").text();
        }

        $("#updateDueDate").replaceWith(document.createTextNode(projection));

        $("#legend table.content-table").remove().insertAfter("table#dueDates").css("width", "100px");

        $("tbody#dueDatesContent td.dueDate:not(.current), tbody#dueDatesContent td.priority:not(.current), tbody#dueDatesContent td.duration:not(.current)").hide();
        
        // Reload html to fix ff2 rendering bug.
        // Not sure which browsers and engine versions have the bug so reloading is done for all platforms.
        $(".ff2reloadfix").each(function(){
            $(this).html($(this).html());
        });

    } else {;
        TW.initCPTableHandlers();
        TW.notifyDiagram({action: "reload"});
    }
    TW.pageData.loadingDialog.hide();
    
    dojo.addOnLoad(handleBidi);
};

/**
 * Checks if historical scenario is selected.
 */
TW.isProjectionSelected = function() {
    return !($("select#path").val() == 3 && TW.pageData.hasId == 0);
};

/**
 * Checks if user changed any value.
 */
TW.hasChanges = function() {
    return $("#changesBody tr").length > 0;
};

/**
 * Shows or hides Historical Scenario drop-down depending on projection selection.
 * If Historical projection is selected scenario drop-down is shown. 
 */
TW.switchHistoricalScenarioVisibility = function() {
    if ($("select#path").val() == 3) {
        $("select#pathHistoricalScenario").show().val(TW.pageData.hasId);
    } else {
        $("select#pathHistoricalScenario").hide();
    }
};

/**
 * Initializes event handlers on page load.
 */
TW.initHandlersOnPageLoad = function() {
    
    // Projection change and Historical Scenario hide/show.
    $("select#path").change(function() {

        if (TW.isProjectionSelected()) {

            if (TW.hasChanges() && !confirm(TW.message.changesWillBeReset)) {
                $("select#path").val(TW.pageData.cpType);
            } else {
                TW.pageData.cpType = $("select#path").val();
                TW.sendAjaxRequest({action: "PREVIEW"}, TW.updateDefaultsCallback);
            }
        }
        
        TW.switchHistoricalScenarioVisibility();
    });

    
    // Historical scenario selection. 
    $("select#pathHistoricalScenario").change(function() {
        TW.pageData.hasId = $("select#pathHistoricalScenario").val();
        $("select#path").change();
    });
    
    // Show current values arrow. 
    $("#showCurrentArrow").click(function() {
        $(".current").show();
        $(this).hide();
        $(this).attr("aria-expanded","false");//a11y
    });

    // Show current values arrow. 
    $("#showCurrentArrow").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {
            $(".current").show();
            $(this).hide();
            $(this).attr("aria-expanded","false");//a11y
        }
    });

    // Hide current values arrow.
    $("#hideCurrentArrow").click(function(e) {
        $("#showCurrentArrow").show();
        $(".current").hide();
        $(this).attr("aria-expanded","true");//a11y
    });

    // Hide current values arrow.
    $("#hideCurrentArrow").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {
            $("#showCurrentArrow").show();
            $(".current").hide();
            $(this).attr("aria-expanded","true");//a11y
        }
    });

    // Show/hide legend.
    $("img#showLegend").click(function() {
        $("div#legend").toggle();
    });
    
    // Show/hide legend.
    $("img#showLegend").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {        
            $("div#legend").toggle();
        }
    });

    // Critical Path only enable/disable.
    $("div#showCritical").click(function() {
        TW.setCriticalMode(!TW.isCriticalMode());
    });

    // Critical Path only enable/disable.
    $("div#showCritical").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) { 
            TW.setCriticalMode(!TW.isCriticalMode());
        }
    });

    // Collapse/expand changes list.
    $("table#changes div.icon").click(function() {
        if ($(this).is(".collapsed")) {
            $(this).removeClass("collapsed");
            $(this).attr("aria-expanded","true");//a11y
            $("#changesBody").children().show();
            $("table#changes div.icon span").html("&#9660");
        } else {
            $(this).addClass("collapsed");
            $(this).attr("aria-expanded","false");//a11y
            $("#changesBody").children().hide();
            $("table#changes div.icon span").html("&#9658");
        }
    });

    // Collapse/expand changes list.
    $("table#changes div.icon").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {
            if ($(this).is(".collapsed")) {
                $(this).removeClass("collapsed");
                $(this).attr("aria-expanded","true");//a11y
                $("#changesBody").children().show();
                $("table#changes div.icon span").html("&#9660");
            } else {
                $(this).addClass("collapsed");
                $(this).attr("aria-expanded","false");//a11y
                $("#changesBody").children().hide();
                $("table#changes div.icon span").html("&#9658");
            }
        }
    });

    // On save.
    $("button#save").click(function() {
        TW.sendAjaxRequest({action: "UPDATE", updateData: TW.getChangesJson()}, TW.handleSave);
    });

    // On revert.
    $("button#revert").click(TW.resetChanges);

    $("#printable").click(function(event) {
        event.preventDefault();
        var url = "criticalPath.do?cpType=" + TW.pageData.cpType;
        url += "&bpdInstanceId=" + TW.pageData.bpdInstanceId;
        if (TW.pageData.hasId) {
            url += "&hasId=" + TW.pageData.hasId;
        }
        url +="&printable=true";
        window.open(url, "printable");
    });
};

/**
 * Changes entered by user in JSON format.
 */
TW.getChangesJson = function() {
    var updates = {};
    $("tbody#changesBody > tr").each(function() {
        
        var itemClass = $(this).attr("tw:itemClass");
        
        if (!updates[itemClass]) {
            updates[itemClass] = {};
        }
        
        updates[itemClass][$(this).attr("tw:itemId")] = 
        {
            priority: $(this).attr("tw:priority"),
            duration: TW.durationObjectToString( TW.parseDurationFromFormFields($(this).attr("tw:duration")) ), //$(this).attr("tw:duration"),
            dueDate : TW.convertToStandardIfNecessary($(this).attr("tw:dueDate"))
        };
        if (itemClass == "activity") {
            updates[itemClass][$(this).attr("tw:itemId")]["prevDueDate"] = TW.convertToStandardIfNecessary($(this).attr("tw:prevDueDate"));
        }
    });
    
    return JSON.stringify(updates);
};

TW.convertToStandardIfNecessary = function(str) {
    var defStr = str;
    if (getCalendarType() == "hebrew") {
        console.log("TW.convertToStandardIfNecessary from: "+str);
        // converts string representation of Hebrew date to Date object
        var hebrewDateObj = dojox.date.hebrew.locale.parse(str, {datePattern: TW.pageData.dateFormat, selector: "date"});
        if (hebrewDateObj) {
            var gregorianDateObj = hebrewDateObj.toGregorian();
            defStr = dojo.date.locale.format(gregorianDateObj, {datePattern:"MM-dd-yyyy HH:mm:ss", selector: "date"});//must match BidiUtils.getStandardDateTimeFormat
            console.log("TW.convertToStandardIfNecessary   to: "+defStr);
        } else {
            console.log("TW.convertToStandardIfNecessary unable to parse hebrew date string to date object");
        }
    }
    if (dojo.locale.toLowerCase() == "pl" && getCalendarType() == "gregorian") {
        var dateObj = dojo.date.locale.parse(str, {datePattern: TW.pageData.dateFormat, selector: "date"});
        if (dateObj) {
            defStr = dojo.date.locale.format(dateObj, {datePattern:"dd.MM.yyyy h:mm:ss", selector: "date"});//must match DateFormatUtils.getDateFormat
            console.log("TW.convertToStandardIfNecessary  pl to: "+defStr);
        } else {
            console.log("TW.convertToStandardIfNecessary unable to parse polish date string to date object");
        }
    }
    return defStr;
};

TW.requestPreview = function() {
    // Don't call preview if user entered incorrect value.
    if (TW.hasErrors(".modified")) { return; };
    
    // Skip update if Mass Update in progress.
    if (TW.pageData.massUpdate) { return; };
    
    TW.sendAjaxRequest({action: "PREVIEW", updateData: TW.getChangesJson()}, TW.previewCallback);
};

// Resets server generated changes. Does nothing to user entered values.
TW.resetPreview = function() {
    $(".preview-overdue").removeClass("preview-overdue");
    $(".preview-atRisk").removeClass("preview-atRisk");
    $(".preview-onTime").removeClass("preview-onTime");
    
    $(".preview").removeClass("preview").defVal(null);
    
    $(".error").filter(".preview").removeClass("error");
    
    $(".wasNonCritical").removeClass("wasNonCritical").removeClass("critical").each(function(){
        if (TW.isCriticalMode()) {
            $(this).hide();
        }
    });

    $(".wasCritical").removeClass("wasCritical").addClass("critical").each(function(){
            $(this).not(".hidden").show();
    });
    
    // Reset projected due date to default.
    $("#projectedDueDate").filter(function() {return $(this).attr("tw:default");}).each(function() {
        $("#projectedDueDate").text($("#projectedDueDate").attr("tw:default"));
    });
};

/**
 * Resets user changes.
 */
TW.resetChanges = function() {
    TW.pageData.loadingDialog.show();
    
    TW.resetPreview();

    $("tbody#dueDatesContent tr").each(function() {
        if ($(this).attr("tw:prevDueDate")) {
            $(this).attr("tw:prevDueDate", $(this).attr("tw:currentPrevDueDate"));
            $(this).attr("tw:currentPrevDueDate", "");
        }
    });

    $(".modified").removeAttr("aria-describedby");
    $(".modified").removeClass("modified").defVal(null);
    $(".error").attr("aria-invalid","false");
    $(".error").removeAttr("aria-describedby");    
    $(".error").removeClass("error");
    
    $("table#changes").hide();
    $(".changeControl").attr("disabled", "disabled");
    $("tbody#changesBody").empty();
    
    // Notify server to recalculate CP for Diagram.
    TW.sendAjaxRequest({action: "REFRESH"}, function() {TW.notifyDiagram({action: "reload"});});
    
    TW.pageData.loadingDialog.hide();
};

/**
 * Loads new defaults on projection change or save.
 * @param updates
 */
TW.updateDefaultsCallback = function(updates) {
    TW.resetChanges();
    
    var id;
    
    for (var type in updates) {
        if (type == "instance") {
            id = TW.pageData.bpdInstanceId;

            if (updates[type][id].projectedDueDate) {
                $("table#instanceDetails #dueDate").text(updates[type][id].dueDate);
                $("table#instanceDetails td.dueDate :text").val(updates[type][id].dueDate);
                $("table#instanceDetails td.dueDate :text").defVal(updates[type][id].dueDate);
                $("#projectedDueDate").attr("tw:default", "");
                $("#projectedDueDate").text(updates[type][id].projectedDueDate);
                
                if (updates[type][id].analysis) {
                    $("#projectedDueDate").addClass(updates[type][id].analysis);
                    $("#projectedDueDate").attr("aria-describedby",updates[type][id].analysis);
                }

                if (updates[type][id].dueDateAnalysis) {
                    $("#dueDate").addClass(updates[type][id].dueDateAnalysis);
                    $("#dueDate").attr("aria-describedby",updates[type][id].dueDateAnalysis);
                } else {
                    $("#dueDate").removeAttr("aria-describedby");
                    $("#dueDate").removeClass("overdue");
                }
            }
        } else {
            for(id in updates[type]) {
                $("tbody#dueDatesContent tr." + type).filter(function() {return $(this).attr("tw:id") == id; }).each(function() {
                    
                    $(this).children("td.duration").children(":text").defVal(updates[type][id].duration);
                    $(this).children("td.dueDate").children(":text").defVal(updates[type][id].dueDate);
                    $(this).children("td.priority").children("select").defVal(updates[type][id].priority);

                    if (updates[type][id].prevDueDate) {
                        $(this).attr("tw:currentPrevDueDate", "");
                        $(this).attr("tw:prevDueDate", updates[type][id].prevDueDate);
                    }

                    $(this).children("td.duration.current").text(updates[type][id].duration);
                    $(this).children("td.dueDate.current").text(updates[type][id].dueDate);
                    
                    if (type === "activity") { 
                        if (updates[type][id].dueDateCalculated === true) {
                            $(this).children("td.dueDate.current").addClass("calculated");
                            $(this).children("td.duration.current").removeClass("calculated");
                        } else if (updates[type][id].dueDateCalculated === false) {
                            $(this).children("td.dueDate.current").removeClass("calculated");
                            $(this).children("td.duration.current").addClass("calculated");
                        }
                    }

                    // Hide non-critical items, show critical.
                    if(updates[type][id].critical) {
                        $(this).addClass("critical");
                        $(this).not(".hidden").show();
                    } else {
                        $(this).removeClass("critical");
                        if (TW.isCriticalMode()) $(this).hide();
                    }
                });
            }
        }
    }

    TW.pageData.loadingDialog.hide();

    TW.notifyDiagram({action: "reload"});

};

TW.handleSave = function(updates) {
    TW.updateDefaultsCallback(updates); 

    // publish general save notification with associated updates
    // we reference the dojo from our parent window to handle typical
    // iframe scenario.  This works even if we are in the top window
    // since window.parent will reference window in this case.
    window.parent.dojo.publish("com.ibm.bpm.dashboards/criticalPath/save", 
		               updates);
};

/**
 * Called when server returns preview data.
 * @param updates
 */
TW.previewCallback = function(updates) {

    TW.resetPreview();

    var id;
    
    for (var type in updates) {
        if (type == "instance") {
            id = TW.pageData.bpdInstanceId;

            if (updates[type][id].projectedDueDate) {
                $("#projectedDueDate").attr("tw:default", $("#projectedDueDate").text());
                $("#projectedDueDate").text(updates[type][id].projectedDueDate);
            
                if (updates[type][id].analysis) {
                    $("#projectedDueDate").addClass("preview-" + updates[type][id].analysis);
                } else {
                    $("#projectedDueDate").addClass("preview-onTime");
                }

            } else {
                assert(false, "Server returned NULL projected due date. This should never happen. \n" +
                              "This message here is to track TW-6033");
            }
            
        } else {
            for(id in updates[type]) { // tasks and activities.
                $("tbody#dueDatesContent tr." + type).filter(function() {return $(this).attr("tw:id") == id; }).each(function() {

                    var newDuration = updates[type][id].duration;
                    
                    if (newDuration && newDuration != $(this).children("td.duration").children(":text").val()) {
                        var control = $(this).children("td.duration").children(":text");
                        control.addClass("preview").val(newDuration);
                        
                        if (type === "task" || newDuration === "" || TW.isDurationValid(newDuration)) {
                                $(control).removeClass("error"); 
                                $(control).attr("aria-invalid","false");
                            } else {
                                $(control).addClass("error");
                                $(control).attr("aria-invalid","true");
                            }
                    }

                    var newDueDate = updates[type][id].dueDate;
                    if (newDueDate && newDueDate != $(this).children("td.dueDate").children(":text").val()) {
                        $(this).children("td.dueDate").children(":text").addClass("preview").val(newDueDate);
                    }

                    var newPriority = updates[type][id].priority;
                    if (newPriority &&  updates[type][id].priority != $(this).children("td.priority").children("select").val()) {

                        $(this).children("td.priority").children("select").addClass("preview").val(updates[type][id].priority);
                    }

                    if (updates[type][id].prevDueDate) {
                        if (!$(this).attr("tw:currentPrevDueDate")) {
                            $(this).attr("tw:currentPrevDueDate", $(this).attr("tw:prevDueDate"));
                        }
                        $(this).attr("tw:prevDueDate", updates[type][id].prevDueDate);
                    }

                    $(this).filter(".critical").addClass("wasCritical");
                    $(this).filter(":not(.critical)").addClass("wasNonCritical");

                    // Show/hide if critical
                    if(updates[type][id].critical) {
                        $(this).addClass("critical");
                        $(this).not(".hidden").show();
                    } else {
                        $(this).removeClass("critical");
                        
                        if (TW.isCriticalMode() && ($(this).find(".modified").length === 0) && ($(this).find(".error").length === 0)) {
                            $(this).hide();
                        } else {
                            TW.setCriticalMode(false);
                        }
                    }
                });
            }
        }
    }

    TW.notifyDiagram({action: "reload"});
    
    TW.pageData.loadingDialog.hide();
};

/**
 * Executed on data reload. Rebinds all listeners.
  */
TW.initCPTableHandlers = function() {
    
    // Hide non-critical items is ON for data loaded from server. 
    $("div#showCritical").addClass("criticalMode");

    // Selected row highlighting.
    $("tbody#dueDatesContent tr").click(function(){
        if ($(this).hasClass("highlighted")) {
            // Deselect row.
            $(this).removeClass("highlighted")
                    .removeClass("highlighted_milestone"); // ie6
            TW.selectOnDiagram();
        } else {
            // Select row.
            $("tbody#dueDatesContent tr.highlighted").removeClass("highlighted")
                    .removeClass("highlighted_milestone"); // ie6
            $(this).addClass("highlighted");
            // ie6
            if ($(this).is(".milestone")) {
                $(this).addClass("highlighted_milestone");
            }
            // If diagram window exists notify it.
            TW.selectOnDiagram(this);
        }
    });

    // Bind collapsing handler.
    $("tbody#dueDatesContent > tr.expandable > td.name > div.icon").click(function(event) {
        TW.collapseClick($(this).parent().parent());
        // And cancel bubble to prevent row selection.
        event.stopPropagation();
    });

    // Bind collapsing handler.
    $("tbody#dueDatesContent > tr.expandable > td.name > div.icon").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {
            TW.collapseClick($(this).parent().parent());
            // And cancel bubble to prevent row selection.
            e.stopPropagation();
        }
    });

    // Cancel bubble on input controls to prevent row selection if control is clicked.
    $("table#dueDates :input, table#instanceDetails :input").click(function(event){
        event.stopPropagation();
    });

    // Cancel bubble on input controls to prevent row selection if control is clicked.
    $("table#dueDates :input, table#instanceDetails :input").keypress(function(e) {
        if (e.which == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {
            e.stopPropagation();
        }
    });

    // Bind text input field change listeners.
    $("tbody#dueDatesContent :text, tbody#dueDatesContent select, table#instanceDetails :text").change(function() {
        var itemClass;
        var itemId;
        var valueClass;
        var newValue;

        var parent = $(this).parent().parent();
        $(this).removeClass("preview");

        // Item class.
        if (parent.hasClass("activity")) {
            itemClass = "activity";
            itemId = parent.attr("tw:id");
        } else if (parent.hasClass("task")) {
            itemClass = "task";
            itemId = parent.attr("tw:id");
        } else {
            itemClass = "instance";
            itemId = TW.pageData.bpdInstanceId;
        }

        // Value class.
        if ($(this).parent().hasClass("duration")) {
            valueClass = "duration";
        } else if ($(this).parent().hasClass("dueDate")) {
            valueClass = "dueDate";
        } else if ($(this).parent().hasClass("priority")) {
            valueClass = "priority";
        } else {
            assert(false, "This should never happen. Invalid value class entered.");
        }

        // Validate and format value.
        newValue = $(this).val();
        $(this).removeClass("error");
        $(this).attr("aria-invalid","false");
        
        if (TW.debug && newValue == "") {
            // Only for debug mode. Reset value to default if it is empty.
            $(this).defVal(null);
            newValue = $(this).defVal(); 
        } else {
            
            switch(valueClass) {
            
                case "dueDate":
                    var checkedDate = TW.checkDueDate($(this).val());
                        
                    if (checkedDate) {
                        newValue = checkedDate;
                        $(this).val(checkedDate);
                    } else {
                        $(this).addClass("error");
                        $(this).attr("aria-invalid","true");
                        newValue = $(this).val();
                    }
                    break;
            
                case "duration":
                    if (TW.isDurationValid(newValue)) {
                        newValue = TW.parseDurationFromUserInput(newValue);
                        var str = TW.durationObjectToString(newValue);
                        $(this).val(str);
                    } else {
                        if ($(this).defVal() === "" && newValue === "") {
                            // To handle activities with custom due dates.
                            // Accept empty string as duration if it is current value.
                        } else {
                            $(this).addClass("error");
                            $(this).attr("aria-invalid","true");
                        }
                    }
                    
                    break;
            
                case "priority":
                    // No validation.
                    break;
            
                default:
                    assert(false, "This should never happen. Unknown value class [" + valueClass + "].")        
            }
        }
        
        var itemName = parent.children("td.name").children("span").text();

        TW.logChange($(this), itemClass, itemId, valueClass, newValue, itemName);

        TW.requestPreview();
    });

    // Select duration text input field on focus.  
    $("td.duration").children(":text").focus(function() {
        this.select();
    });
    
    // Bind calendar to all due date editors.
    TW.initCalendar();
};


/**
 * Validates due date and returns date if it is complies format and is in future. 
 * @param stringDate date to parse.
 */
TW.checkDueDate = function(stringDate) {
    
    var date = TW.parseDateWithFailure(stringDate, TW.pageData.dateFormat);
    if (date && date > new Date()) {        
    // Need to set en as locale here since only English input date is supported.  We can remove the locale if we support non-English input date.
		return getDojoDatePackage().locale.format(date, {datePattern: TW.pageData.dateFormat, selector: "date"});
    } else {
        return null;
    }
};

/**
 * Selects or deselects item on diagram.
 * @param node tr to select.
 */
TW.selectOnDiagram = function(node) {
    var params = {};

    if (node) {

        // If task is selected diagram highlights its activity.
        var idToSelect = $(node).is(".task") ? $(node).attr("tw:parent") : $(node).attr("tw:id");

        idToSelect = TW.trimmParentFromGUID(idToSelect);

        params = {
            action     : "select",
            idToSelect : idToSelect,
            stack      : buildCallStack(node)
        };
        
    } else {
        params = { action : "deselect" };
    }

    TW.notifyDiagram(params);

    // --------- local functions --------------

    /**
     * Recursively walks up the tree building call stack.
     * @param startNode Node to start from.
     * @param stack Array representing stack.
     */
    function drillUpCallStack(startNode, stack) {
        $("tbody#dueDatesContent tr").filter(function() {return $(startNode).attr("tw:parent") == $(this).attr("tw:id");}).each(function() {
            if ($(this).attr("tw:subbpdid")) {
                var subBpd = {
                    "bpdId"    : $(this).attr("tw:subbpdid"),
                    "parentId" : $(this).attr("tw:id"),
                    "name"     : $(this).find("td.name").text()
                };
                stack.unshift(subBpd);
            }
            drillUpCallStack($(this), stack);
        });

    };

    /**
     * Gets call stack including top level BPD.
     * @param startNode Node to start with.
     */
    function buildCallStack(startNode) {
        var stack = [];
        drillUpCallStack(startNode, stack);
        var rootBpd = {
            "bpdId" : TW.pageData.bpdId,
            "name"  : TW.pageData.bpdName
        };
        stack.unshift(rootBpd);
        return stack;
    }
};

/**
 * Collapse click listener.
 * @param rowClicked Row.
 */
TW.collapseClick = function(rowClicked) {
    var expanding = rowClicked.hasClass("collapsed");
    if (expanding) {    
        $("tbody#dueDatesContent > tr.expandable > td.name > div.icon span").html("&#9660");
    } else {
        $("tbody#dueDatesContent > tr.expandable > td.name > div.icon span").html("&#9658");
    }
    collapseChildren(rowClicked, expanding);

    // --------- local functions ---------------
    
    /**
     * Recursively collapses children traversing the tree.
     * @param rowClicked
     * @param expanding True if expanding, False otherwise.
     */
    function collapseChildren(rowClicked, expanding) {
        if (expanding) {
            $(rowClicked).filter(".expandable").removeClass("collapsed");
            $(rowClicked).filter(".milestone").removeClass("milestone_collapsed"); // ie6
            $(rowClicked).filter(".activity").removeClass("activity_collapsed"); // ie6
        } else {
            $(rowClicked).filter(".expandable").addClass("collapsed");
            $(rowClicked).filter(".milestone").addClass("milestone_collapsed"); // ie6
            $(rowClicked).filter(".activity").addClass("activity_collapsed"); // ie6
        }
        
        var id = $(rowClicked).attr("tw:id");
        
        $("tbody#dueDatesContent > tr").filter(function() {return $(this).attr('tw:parent') == id; }).each(function() {
            if (expanding) {
                // Expanding.
                if (TW.isCriticalMode()) {
                    $(this).removeClass("hidden")
                            .filter(".critical").show()
                            .filter(":not(.expandable)").removeClass("collapsed")
                            .removeClass("activity_collapsed").removeClass("milestone_collapsed"); // ie6
                } else {
                    $(this).removeClass("hidden").show()
                            .filter(":not(.expandable)").removeClass("collapsed")
                            .removeClass("activity_collapsed").removeClass("milestone_collapsed"); // ie6
                }
            } else {
                // Collapsing.
                $(this).hide().addClass("hidden");
                collapseChildren($(this), false);
            }
        });
    };
};


/**
 * Initialize calendar controls.
 */
TW.initCalendar = function() {

    var idGenerator = 0;
    
    var calendarPopup = new com.lombardisw.DateTimePopup({});

    $("input[type='text'].needJscalendar").each(function() {
        // Generate ID for every due date editor with no ID.
        var fieldId;
        if ($(this).attr("id")) {
            fieldId = $(this).attr("id");
        } else {
            fieldId = "cal-" + idGenerator++ + "-dueDate";
            $(this).attr("id", fieldId);
            $(this).attr("aria-label",TW.message.dueDate);            
        }
        $(this).attr("name", fieldId);

        // Append image to every due date editor.
        var button = document.createElement('img');
        $(button).attr("src", "images/calendar_16x16.gif");
        $(button).css("vertical-align", "middle");
        $(button).css("margin-left", "1ex");
        $(button).attr("id", fieldId + "-button");
        $(button).attr("tabindex","0");
        $(button).attr("role","button");
        $(button).attr("aria-label",TW.message.calendar);
        $(button).attr("alt","");
        $(button).addClass("jscalendar-icon");
        $(this).after(button);

        $(this).parent().addClass("jscalendar");

        var tf = $(this);
        dojo.connect(button, "onclick", function(e){
            calendarPopup._pushChangeTo = tf;
            calendarPopup.attr("value", getDojoDatePackage().locale.parse(tf.attr("value"), {datePattern: TW.pageData.dateFormat, selector: "date"}));
            dijit.popup.open({ 
                popup: calendarPopup,
                around: dojo.byId(tf.attr("id"))
            });
            // Only way I can get popup to disappear when mouse clicks outside
            calendarPopup.doFocus();
        })
        dojo.connect(button, "onkeypress", function(e){
            if (e.charOrCode == dojo.keys.ENTER || e.which == dojo.keys.SPACE) {            
                calendarPopup._pushChangeTo = tf;
                // Need to set en as locale here since only English input date is supported.  We can remove the locale if we support non-English input date.
                calendarPopup.attr("value", getDojoDatePackage().locale.parse(tf.attr("value"), {datePattern: TW.pageData.dateFormat, selector: "date"}));
                dijit.popup.open({ 
                    popup: calendarPopup,
                    around: dojo.byId(tf.attr("id"))
                });
                // Only way I can get popup to disappear when mouse clicks outside
                calendarPopup.doFocus();
            }
        }) 
        
        $(this).removeClass("needJscalendar").addClass("jscalendar");
        
    });
};

// Stores change in changes log.
TW.logChange = function(control, itemClass, itemId, valueClass, newValue, name) {
    
    // Cache row of critical path table having change.
    var changedCPTableRow = control.parent().parent();

    // Look for existing change for this item.
    var logRows = $("tbody#changesBody").children("tr").filter(function() {
        return $(this).attr('tw:itemId') == itemId && $(this).attr("tw:itemclass") === itemClass;});

    // Check if there is existing record for this item.
    if (logRows.length > 0) {
        // If there is a record.
        var logRow = $(logRows[0]);

        // If duration is set due date must be reset and vise-versa.
        if (valueClass == "duration" || valueClass == "dueDate") {
            logRow.removeAttr("tw:duration").removeAttr("tw:dueDate");

            var editorToReset;

            if (valueClass == "duration") {
                editorToReset = changedCPTableRow.find("td.dueDate :text");

            } else if (valueClass == "dueDate" && itemClass == "activity") {
                editorToReset = changedCPTableRow.find("td.duration :text");
            }

            if (editorToReset) {
                editorToReset.removeClass("modified").removeClass("error").defVal(null);
                editorToReset.removeAttr("aria-describedby");
                editorToReset.attr("aria-invalid","false");
            }
        }

        // Check if value is the same as default.
        if (newValue != $(control).defVal()) {
            // Save if value is different.
            if(valueClass === "duration") {
                logRow.attr("tw:" +valueClass, TW.durationObjectToString(newValue));
            } else {
                logRow.attr("tw:" + valueClass, newValue);
            }
            control.addClass("modified");
            control.attr("aria-describedby",TW.message.valueModified);
        } else {
            // Reset change if new value is the same as default.
            logRow.removeAttr("tw:" + valueClass);
            control.removeClass("modified");
            control.removeAttr("aria-describedby");
        }

        // Check if there are any changes for the item.
        if (    typeof logRow.attr("tw:duration") !== 'undefined' 
             || typeof logRow.attr("tw:dueDate")  !== 'undefined' 
             || typeof logRow.attr("tw:priority") !== 'undefined') {
            // If there is any data update message.
            logRow.find("td.changeText").text(buildChangeText(logRow, changedCPTableRow));
        } else {
            // If record does not have any data delete it.
            logRow.remove();
        }

    } else {
        // There is no record for this item.

        // Check if new value is the same as current.
        if (newValue != $(control).defVal()) {
            
            // Create new change log entry if it is different.
            var newLogEntry = document.createElement("tr");
            $(newLogEntry).attr("tw:itemClass", itemClass);
            $(newLogEntry).attr("tw:name", name);
            $(newLogEntry).attr("tw:itemId", itemId);
            if(valueClass === "duration") {
                $(newLogEntry).attr("tw:" +valueClass, TW.durationObjectToString(newValue));
            } else {
                $(newLogEntry).attr("tw:" + valueClass, newValue);
            }

            if (itemClass === "activity") {
                $(newLogEntry).attr("tw:prevDueDate", changedCPTableRow.attr("tw:prevDueDate"));
            }

            var text = document.createElement("td");
            $(text).text(buildChangeText(newLogEntry, changedCPTableRow)).addClass("changeText");
            
            newLogEntry.appendChild(text);
            
            $('tbody#changesBody').append(newLogEntry);

            control.addClass("modified");
            control.attr("aria-describedby",TW.message.valueModified);
            
        } else {
            control.removeClass("modified");
            control.removeAttr("aria-describedby");
        }
    }

    showHideChangesPane();

    // -------------------- Local functions -------------

    /**
     * Constructs change text.
     * @param changeRow
     * @param cpTableRow
     */
    function buildChangeText(changeRow, cpTableRow) {
	   
        var changedClassName = $(changeRow).attr("tw:itemClass");
        var oldDueDate = cpTableRow.find("td.dueDate :text").defVal();

        if (changedClassName == "activity") {
            var oldDuration = cpTableRow.find("td.duration :text").defVal();
        }
        if (changedClassName == "task" || changedClassName == "activity") {
            
            var priorityDropDown = cpTableRow.find("td.priority select");
            var newPriorityName = priorityDropDown.find("option:selected").text();
            
            var oldPriorityName = priorityDropDown.find("option[value=" + priorityDropDown.defVal() + "]").text();
        }
        
		var changeRowName = $(changeRow).attr("tw:name");
		
		var message = "";
		
        if ($(changeRow).attr("tw:priority")) {		
            
        if(changedClassName == "activity") {
	        message += $("div#priorityModificationActivity").text();
		}
			
	    if(changedClassName == "task") {
	        message += $("div#priorityModificationTask").text();
		}
		
        if(changedClassName == "instance") {
	        message += $("div#priorityModificationInstance").text();
	    }			
		
   		    message = message.replace("{0}", changeRowName);
			message = message.replace("{1}", oldPriorityName);
			message = message.replace("{2}", newPriorityName);
        }

        var newDuration = $(changeRow).attr("tw:duration");
		var newDueDate = TW.convertToStandardIfNecessary($(changeRow).attr("tw:dueDate"));		
	    if (typeof newDuration !== 'undefined') {
			// newDuration might be 'null'
			if (newDuration=="null") {
				message += $("div#durationModificationInvalid").text();
			} else {
				var str = newDuration;
				updatedDuration = str;

				str = str.split(" ");
				
				if(str[1] == "hour")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#hour").text());
				else if(str[1] == "minute")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#minute").text());
				else if(str[1] == "day")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#day").text());
				else if(str[1] == "month")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#month").text());
				else if(str[1] == "hours")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#hours").text());
				else if(str[1] == "minutes")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#minutes").text());
				else if(str[1] == "days")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#days").text());
				else if(str[1] == "months")		  
				updatedDuration = updatedDuration.replace(str[1],$("div#months").text());
					
				updatedDuration = updatedDuration.replace(/^\s*/, '').replace(/\s*$/, '');
				
				if(changedClassName == "activity") {
					message += $("div#durationModificationActivity").text();
				}
				
				if(changedClassName == "task") {
					message += $("div#durationModificationTask").text();
				}
			
				if(changedClassName == "instance") {
					message += $("div#durationModificationInstance").text();
				}          
				 message = message.replace("{0}", changeRowName);
				 message = message.replace("{1}", oldDuration);
				 message = message.replace("{2}", updatedDuration);  
			}
        } else if (typeof newDueDate !== 'undefined') {
		
		  if(changedClassName == "activity") {
	            message += $("div#dueDateModificationActivity").text();
		  }
			
	      if(changedClassName == "task") {
	            message += $("div#dueDateModificationTask").text();
		  }
		
		  if(changedClassName == "instance") {
		        message += $("div#dueDateModificationInstance").text();
	      }
            message = message.replace("{0}", changeRowName);
			message = message.replace("{1}", oldDueDate);
			message = message.replace("{2}", newDueDate);
	    }
        
        return message;
    }

    /**
     * Hides changes pane and disables Send/Revert buttons if change list is empty.
     */
    function showHideChangesPane() {
        if ($("table#changes > tbody#changesBody").children().length > 0) {
            $(".changeControl").attr("disabled", "");
            $("table#changes").show();
        } else {
            $(".changeControl").attr("disabled", "disabled");
            $("table#changes").hide();
        }
    }
};

/**
* Parses the duration from user input. This method will only be called
* to check the user's input
* Returns null if it can't parse
*/
TW.parseDurationFromUserInput = function(inputString) {
    if (!inputString) return null; // no input provided -- return null to indicate invalid input
    
    // Parse the duration string
    var match = TW.parseDurationString(inputString);
    if (match == null || match.length != 3) return null; // Failed to parse -- return null to indicate invalid input
    
    var durationValue = +match[1]
    if (durationValue === 0) return null; // Invalid input value

    var returnObject = {};
    returnObject.value = Math.round(durationValue);
    returnObject.units = match[2].toLowerCase();

    var inputDurationUnit = returnObject.units;
    if (inputDurationUnit === "") { // if the user didn't input a duration unit, default to day(s)
         if (TW.isSingular(returnObject.value)) {
            returnObject.units = "day";
         } else {
            returnObject.units = "days";
         }
    }

    if (TW.isEnglishDurationUnit(returnObject.units)) {
        // check in the input duration unit is an english duration unit; if so, translate it
        returnObject.units = TW.getTranslatedDurationUnit(returnObject.units);

        if (!returnObject.units) {
            return null; // We weren't able to find the translated version, something must have gone wrong, return null for error state
        }
    }

    if (TW.isTranslatedDurationUnit(returnObject.units)) {
        // If a valid translated duration unit, then update the returnObject and return it
        return returnObject;
    }

    return null; // Invalid input
};

/**
* Parses duration strings from form fields and converts them 
* to the English equivalent to be sent to the server
*/
TW.parseDurationFromFormFields = function(inputString) {
    if (!inputString) return null; // no input provided -- return null to indicate invalid input

    if (inputString.value && inputString.units) return inputString; // already been parsed

    var match = TW.parseDurationString(inputString);
    if (match == null || match.length != 3) return null; // Failed to parse -- return null to indicate invalid input

    var durationValue = +match[1];
    if (durationValue === 0) return null; // invalid input value

    var durationUnit = match[2].toLowerCase();
    if (durationUnit === "") {
        if (TW.isSingular(result.value)) {
            durationUnit = "day";
        } else {
            durationUnit = "days";
        }
    }

    if (TW.isTranslatedDurationUnit(durationUnit)) {
        // Translate the duration unit back to English
        durationUnit = TW.getEnglishDurationUnit(durationUnit);

        var returnObject = {
            value: durationValue,
            units: durationUnit
        };
        return returnObject;
    }

    return null;
};

/**
* Converts a translated object into it's English duration unit equivalent
*/
TW.convertTranslatedToEnglishDurationObject = function(translatedObject) {
    if (TW.isEnglishDurationUnit(translatedObject.units)) {
        // it's already an english unit, so no need to continue
        return translatedObject;
    }

    var tmp = TW.translatedToEnglishDurationUnits[translatedObject.units];
    translatedObject.units = tmp;

    return translatedObject;
};

/**
* Returns a string representation of a duration object
* Object is of the form 
* {
*    value: durationValue
*    units: durationUnit
* }
*
* Return string is of the form
* "[durationValue] [durationUnit]"
* e.g., "2 days"
*/
TW.durationObjectToString = function(obj) {
    if (!obj || !obj.value || ! obj.units) {
        return null; // invalid duration unit object
    }
    return obj.value +" " +obj.units;
}

/**
* Returns the English version of a translated duration unit
*/
TW.getEnglishDurationUnit = function(durationUnit) {
    if (TW.translatedToEnglishDurationUnits[durationUnit]) {
        return TW.translatedToEnglishDurationUnits[durationUnit];
    }

    return null;
};

/**
* Returns the translated version of the english duration unit
*/
TW.getTranslatedDurationUnit = function(englishDurationUnit) {
    return TW.translatedDurationUnits[englishDurationUnit];
};

/**
* Whether or not the duration unit is an english duration unit
*/
TW.isEnglishDurationUnit = function(str) {
    for (var i = 0; i < TW.englishDurationUnits.length; i++) {
        var cur = TW.englishDurationUnits[i];
        if (str === cur) return true;
    }
    return false;
};

/**
* Whether or not the duration unit is a translated duration unit
*/
TW.isTranslatedDurationUnit = function(str) {
    if (TW.translatedToEnglishDurationUnits[str]) {
        return true;
    }
    return false;
};

/**
* Parses the duration string with regular expressions
* To separate the duration value and the duration unit
*/
TW.parseDurationString = function(str) {
    str = str.replace(/\s+/g, ""); // Remove everything except alpha-numerics

    // Parse the duration string
    var regex = new RegExp(TW.buildRegexString());
    return str.match(regex);
}

/**
* Creates a regex to parse the duration string
*/
TW.buildRegexString = function() {
    var regexString = "^(\\d+)(";
    for (var i = 0; i < TW.englishDurationUnits.length; i++) {
        var cur = TW.englishDurationUnits[i];
        regexString += cur +"|"; // add the english duration unit
        regexString += TW.translatedDurationUnits[cur] +"|"; // add the translated duration unit
    }
    regexString += ")$";
    return regexString;
};

/**
 * Parses duration from string.
 * Returns null if can't parse.
 * Accepts values like "1d", "1 day", "5 month" etc.
 * If no units supplied will assume days.
 */
TW.parseDuration = function(str) {
    assert(typeof str !== "undefined", "TW.parseDuration needs one parameter.");
    
    if (!str) return null;
    if (str.value && str.units) {
        return str; // passthrough parsed.
    }
    
    var result = {
        value: 0,
        units: "day" // Which units to use if no units given.
    };

    str = str.replace(/\s+/g, ""); // Remove all except alpha-numerics.
    
    var regexStr = "^(\\d+)(month|minute|min|mon|hour|hr|h|day|d|"; //d|h|hr|min|"; //month|minute|min|hour|hr|h|day|d|";
    for (var durationUnit in TW.translatedDurationUnits) {
        regexStr += TW.translatedDurationUnits[durationUnit] +"|";
    }
    regexStr += ")$";
    var regex = new RegExp(regexStr);
    var parts = str.match(regex); // Parse digits immediately following by text.

    //var parts = str.match(/^(\d+)(month|minute|min|hour|hr|h|day|d|)[s]?$/); // Parse digits immediately following by text.
    
    if (parts == null || parts.length != 3) { // Failed to parse -- return null.
        return null;
    }

    result.value = +parts[1];
    
    if (result.value === 0) {
        return null;
    }
    
    var unparsedUnits = parts[2].toLowerCase();
    
    if (unparsedUnits === "") {
        if (TW.isSingular(result.value)) {
            unparsedUnits = "day";
        } else {
            unparsedUnits = "days";
        }
    }

    // if the unparsedUnit is an abbreviation, replace it with it's proper, spelled-out unit name
    if (unparsedUnits in TW.durationUnitAbbreviations) {
        unparsedUnits = TW.durationUnitAbbreviations[unparsedUnits];
    }

    var englishUnit = TW.getEnglishDurationUnit(unparsedUnits, result.value);
    if (englishUnit) {
        result.units = englishUnit;
        return result;
    }

    return null;
};

/**
 * Formats duration into printable form, including the translation of the duration unit
 * @param duration
 */
TW.formatDuration = function(duration) {
    assert(typeof duration !== "undefined", "TW.formatDuration should have one parameter.");
    if (duration == null) return null;

    var durationValue = Math.round(duration.value);
    var durationUnit = TW.translatedDurationUnits[duration.units];
    
    return durationValue +" " +durationUnit;
};

/**
* Formats duration into printable form, without translating the duration unit
* @param duration
*/
TW.formatDurationWithoutTranslation = function(duration) {
    assert(typeof duration !== "undefined", "TW.formatDuration should have one parameter.");
    if (duration == null) return null;

    var durationValue = Math.round(duration.value);

    return durationValue +" " +duration.units;
};

TW.isSingular = function(value) {
    return value === 1;
};

/**
 * Builds plural form if value is > 1 by appending "s".
 * @param value
 * @param singular
 */
TW.plural = function(value, singular) {
    return value == 1 ? singular : (singular + "s");
};

/**
 * jQuery extension for default values.
 * Works with input[type=text] natively and select (via custom "tw:default" attribute). 
 * Use $(this).defVal() to get default value.
 * Use $(this).defVal(value) to set default value.
 * Use $(this).defVal(null) to reset value to default.
 * @param val
 */
jQuery.fn.defVal = function(val) {

    if (val === undefined) {    // No parameters -- read value.
        if ($(this).length) {
            if ($(this).is(":text")) {
                return $(this).get(0).defaultValue;
            } else if ($(this).is("select")) {
                return  $(this).attr("tw:default");
            }
        }
        return undefined;

    } else if ( this.length ) {
        return this.each(function() {
            if ( this.nodeType != 1 ) return;

            var elem = $(this);

            if (val === null) { // Null parameter -- reset to default value.
                if (elem.is(":text")) {
                    elem.val($(elem).get(0).defaultValue);
                } else if (elem.is("select")) {
                    elem.val(elem.attr("tw:default"));
                }
            } else {                    // Has parameter -- set default value
                if (elem.is(":text")) {
                    elem.get(0).defaultValue = val;
                } else if (elem.is("select")) {
                    elem.attr("tw:default", val);
                }
                elem.val(val);
            }
        });
    }
};

/**
 * Parse date from string using format. Heavily depends on jscalendar.
 * Should maintain the same format as jscalendar.
 * @param str
 * @param fmt
 */
TW.parseDateWithFailure = function(str, fmt) {
	// Need to set en as locale here since only English input date is supported.  We can remove the locale if we support non-English input date.
    return getDojoDatePackage().locale.parse(str, {datePattern: fmt, selector: "date"});
};


/**
 * Called by Flash Diagram popup window on item select.
 * @param param
 */
function diagramToTable(param) {
    
    if (param.action === "select") {
        // Remove selection from all rows.
        $("tbody#dueDatesContent tr.highlighted").removeClass("highlighted")
                .removeClass("highlighted_milestone"); // ie6

        // Find a row with id to select.
        $("tbody#dueDatesContent tr").filter(function() {return TW.trimmParentFromGUID($(this).attr("tw:id")) == param.id;}).each(function() {

            $(this).addClass("highlighted");
            
            // ie6
            if ($(this).is(".milestone")) {
                $(this).addClass("highlighted_milestone");
            }

            // Expand it if it is.
            if ($(this).is(".hidden")) {
                TW.expandTo($(this));
            }

            // Bring non-critical nodes into view if needed.
            if (TW.isCriticalMode() && $(this).is(":not(.hidden)") && $(this).is(":hidden")) {
                TW.setCriticalMode(false);
            }
        });
        
    } else if (param.action == "onload") {
        if ($("tbody#dueDatesContent tr.highlighted").length != 0) {
            TW.selectOnDiagram($("tbody#dueDatesContent tr.highlighted"));
        }
    }
}

/**
 * Pass parameters to diagram.
 * @param param
 */
TW.notifyDiagram = function(param) {
    // Typeof guard for TFO.
    if (typeof(diagramPopup) != "undefined" && diagramPopup && !diagramPopup.closed && diagramPopup.tableToDiagram) {
        diagramPopup.tableToDiagram(param);
    }
};

/**
 * Expands tree to display node.
 * @param node
 */
TW.expandTo = function(node) {
    $("tbody#dueDatesContent tr").filter(function() {return $(this).attr("tw:id") == $(node).attr("tw:parent");}).each(function(){
        if ($(this).is(".hidden")) {
            TW.expandTo(this);
        }
        if ($(this).is(".collapsed")) {
            $(this).find("td.name > div.icon").click();
        }
    });
};

/**
 * Cuts off parent part of GUID if guid is in format {guid}!{guid}.
 * Does nothing if guid is well-formed.
 * This GUID form is used by server back end to identify which BPD is called by which activity.
 * 
 * @param guid
 */
TW.trimmParentFromGUID = function (guid) {
    var realGUID = guid;
    var indexOfBang = guid.indexOf("!");
    if (indexOfBang >= 0) {
        realGUID = guid.substring(0, indexOfBang);
    }
    return realGUID;
};

/**
 * Checks for errors. By default check for any error field.
 * hasErrors(".modified") will search for errors only in user modified fields.
 * 
 * @param which String to narrow search of errors.
 *  
 */
TW.hasErrors = function(which) {
    if (which) {
        return $(which).filter(".error").length > 0;
    } else {
        return $(".error").length > 0;
    }
};

/**
 * Returns True if "Hide non-critical" mode is enabled.
 */
TW.isCriticalMode = function() {
    return $("div#showCritical").is(".criticalMode");
};

/**
 * Sets critical mode.
 * @param mode
 */
TW.setCriticalMode = function(mode) {
    if (mode != TW.isCriticalMode()) {
        if (mode) {
            $("tbody#dueDatesContent > tr").filter(":not(.critical)").hide();
            $("div#showCritical").addClass("criticalMode");
            $("div#showCritical").attr("title", TW.message.showAllActivities);
            $("div#showCritical img").attr("src", "images/icon/critical-mode.gif");
        } else {
            $("tbody#dueDatesContent > tr:hidden").filter(":not(.critical)").filter(":not(.hidden)").show();
            $("div#showCritical").removeClass("criticalMode");
            $("div#showCritical").attr("title", TW.message.hideNonCriticalActivities);
            $("div#showCritical img").attr("src", "images/icon/non-critical-mode.gif");
        }
    }
};

/**
 * Checks if duration is valid.
 * @param duration Duration to check.
 */
TW.isDurationValid = function(durationToCheck) {
    var duration = TW.parseDurationFromUserInput(durationToCheck);
    return     duration 
            && duration.units 
            && duration.value 
            && duration.value > 0;
};

TW.checkIfDebugModeAndInit = function() {
    
    if (typeof TW.debug === 'undefined') { 
        var url = location.href;
        TW.debug = url.search(/[&?]debug/) >= 0;
        
        // Setup debug.
        if (TW.debug) {
            
            // Setup debug CSS.
            $("head").append( // Blue highlight preview vs current diff. 
                    "<style type='text/css'>" + 
                    "input.preview, select.preview {color: blue;} " +
                     "</style>"
                    );
        }
    }
    
    return TW.debug;
};

/**
 * Throws an exception with message if expression is FALSE.
 */
function assert(expression, message) {
    if (TW.debug) {
        var assertionMessage = message || "Assestion failed";
    
        if (!expression) {
            throw new Error("AssertionError :" + assertionMessage);
        }
    }
}

dojo.provide("com.lombardisw.DateTimePopup");
dojo.require("dijit.Calendar");
dojo.require("dojox.form.TimeSpinner");

dojo.declare("com.lombardisw.TimeSpinner", dojox.form.TimeSpinner, {
	setCalendar: function setCalendar(cal){
		this.calendar = cal;
	},
    _onKeyPress: function(e){ 
         if((e.charOrCode == dojo.keys.HOME || e.charOrCode == dojo.keys.END) && !(e.ctrlKey || e.altKey || e.metaKey) 
			&& typeof this.attr('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){ 
			var value = this.constraints[(e.charOrCode == dojo.keys.HOME ? "min" : "max")]; 
			if(value){ 
				this._setValueAttr(value,true); 
			} 
			// eat home or end key whether we change the value or not 
			dojo.stopEvent(e); 
		}
	},
	isValid: function(bool) {
		if (this.calendar) {
			// refresh the calendar widget with the updated time
			var date = this.calendar.value;
			var newTime = this.get("value");
			if (newTime) {
				if (newTime.getHours()!=null && newTime.getHours()!=undefined && newTime.getHours()!='undefined') {
					date.setHours(newTime.getHours());
				} else {
					return false;
				}
				if (newTime.getMinutes()!=null && newTime.getMinutes()!=undefined && newTime.getMinutes()!='undefined') {
					date.setMinutes(newTime.getMinutes());
				} else {
					return false;
				}
				if (newTime.getSeconds()!=null && newTime.getSeconds()!=undefined && newTime.getSeconds()!='undefined') {
					date.setSeconds(newTime.getSeconds());
				} else {
					date.setSeconds(0);
				}
				this.calendar.attr("value", date);
				this.calendar._populateGrid();
			} else {
				return false;
			}
		}
		return true;
	},
	parse: function(time, locale) {
        console.log("criticalPath parse time: "+time);
        if (time.indexOf("AM") > -1 && getDojoDatePackage() == "dojox.date.hebrew") {
            time = "00:00:00";//temp for debugging
        }
		var displayedValue;
		if (time) {
			displayedValue = time;
		} else {
			displayedValue = this.get("displayedValue");
		}
		var timeRet=null;
		try {
			timeRet = getDojoDatePackage().locale.parse(displayedValue, {datePattern: TW.pageData.timeFormat, selector: "time"});
		} catch (e) {
			if (!timeRet) {
				timeRet = dojo.date.locale.parse(displayedValue, {datePattern: TW.pageData.timeFormat, selector: "time"});
			}
		}
		if (!timeRet) {
			timeRet = new Date();
		}
		if (timeRet && (!timeRet.getTimezoneOffset)) {
			timeRet.getTimezoneOffset = function(){return 0;};
		}
        console.log("criticalPath parse time returning "+timeRet);
        console.log(timeRet);
		return timeRet;
	}
});

dojo.declare("com.lombardisw.DateTimePopup", [dijit._Widget, dijit._Templated], { 
    templateString: '' +
    ' <div>' +
	'		<div dojoAttachPoint="datePicker" ></div>' +
	'		<div dojoAttachPoint="timePicker" ></div></div>',

    constructor: function(args){
    
        var options = {};
        if (args.value) {
            options.value = args.value;
        }
                
        var me = this;
        this.dateWidget = new dijit.Calendar({
        		// due to a bug with hebrew calendar, a calendar widget must always be initialized with a value
        		value: new Date(),
                datePackage: getDojoDatePackageAsString(),
                style: "width:190px",
				isDisabledDate: function(d){
					if (this.value) {
						// disable the dates if they are before the current date.
						var d = new Date(d);
						d.setHours(this.value.getHours());
						d.setMinutes(this.value.getMinutes());
						d.setSeconds(this.value.getSeconds());
						var today = new Date();
						var diff = dojo.date.difference(today, d, "second");
						return diff < 0;
					} else {
						return false;
					}
				},
                _onKeyPress: function(/*Event*/ evt){
                    if (evt.keyCode == dojo.keys.TAB) {
                        dijit.focus(dojo.byId(me.timeWidget.id));
                    } else if(!this.handleKey(evt)){//if key was handled by calendar, stopEvent
                        dojo.stopEvent(evt);
                    }                    
                    if (evt.keyCode == dojo.keys.ESCAPE) {//escape out of calendar popup
                        dojo.stopEvent(evt);
                        dijit.popup.close(me);                        
                    }
                },
                destroy: function(){
                    // avoid leaving a ref in object for IE?
                    delete me._pushChangeTo;
                    this.inherited(arguments);
                }
        });
        dojo.attr(this.dateWidget.domNode,"tabindex","0");
        this.timeWidget = new com.lombardisw.TimeSpinner({
                style: "width:190px",
                _onKeyPress: function(e){ 
                     if (e.keyCode == dojo.keys.TAB) {
                         console.log("time tab to calendar");
                         dijit.focus(dojo.byId(me.dateWidget.id));
                     } else if((e.charOrCode == dojo.keys.HOME || e.charOrCode == dojo.keys.END) && !(e.ctrlKey || e.altKey || e.metaKey) 
                        && typeof this.attr('value') != 'undefined' /* gibberish, so HOME and END are default editing keys*/){ 
                        var value = this.constraints[(e.charOrCode == dojo.keys.HOME ? "min" : "max")]; 
                        if(value){ 
                            this._setValueAttr(value,true); 
                        } 
                        // eat home or end key whether we change the value or not 
                        dojo.stopEvent(e); 
                    }
                     if (e.keyCode == dojo.keys.ESCAPE) {
                         dojo.stopEvent(e);
                         dijit.popup.close(me);
                     }
                }

 	    }); 
 	    this.timeWidget.setCalendar(this.dateWidget);
        dojo.attr(this.timeWidget.domNode,"tabindex","0");
    },

    postCreate: function(){
        this.dateWidget.placeAt(this.datePicker);
        this.timeWidget.placeAt(this.timePicker);
	    this.timeWidget.startup();
	
        var me = this;
	    dojo.connect(this.dateWidget, "onValueSelected", function(val){
              var allVal = me.attr("value");
              if (allVal) {
            	  // Need to set en as locale here since only English input date is supported.  We can remove the locale if we support non-English input date.
                  var fdate = getDojoDatePackage().locale.format(allVal, {datePattern: TW.pageData.dateFormat, selector: "date"});
			      if (fdate && (new Date() <= allVal)) { // Deny dates in the past.
				      dijit.popup.close(me);
				      me._pushChangeTo.attr("value", fdate);
        			  me._pushChangeTo.change();
			      }
	          }
        });
    },
    _onBlur: function() {
        dijit.popup.close(this);
    },
    attr: function attr(name, value){
        var rc = null;	
        try {
          if (name == "value") {
		    if (value == undefined) {
                this.dateWidget.value.setHours(this.timeWidget.value.getHours());
                this.dateWidget.value.setMinutes(this.timeWidget.value.getMinutes());
                this.dateWidget.value.setSeconds(this.timeWidget.value.getSeconds());
                rc = this.dateWidget.value;
		    } else {
                this.timeWidget.attr("value", value); // time part
                rc = this.dateWidget.attr("value", value); // date part
		    }
          }
          return rc;
        } catch(e) {
          return null;
        }
    },
    
    setValue: function(value){
        this.attr("value", value);
    },
    getValue: function(){
        return this.attr("value");
    },
    doFocus: function(){
        this.timeWidget.focus();
    }
});
