/*
 * SVG.js
 *
 * A small amount of common SVG code.
 *
 * Copyright - Lombardi Software 2005
 * 
 * @version 0.1 - 03.09.2005 
 * @author Hugh Leather (hughleat@hotmail.com) 
 */

/**
 * This class holds constants applicable for SVG
 **/
function SVG () {
}
/**
 * The namespace
 */
SVG.ns = "http://www.w3.org/2000/svg";
/**
 * Adobe SVG 3.0 doesn't provide a way to canonicalize lengths.  There are many
 * ways to put lengths in SVG attributes; consider '1em', '20cm' and '4px'.
 * Often, we should like to have a canonical representation that can be modified
 * by ordinary arithmetic.
 * <br/>
 * This function, converts a length (as a string or number) to a user space 
 * length.  A scoping node is required since in particular, the font size may be
 * depend upon it ('1em' means 1*font-size).  The result is a number which is 
 * expressed in the user coordinates of the scope (i.e. it is not in the top-
 * level user coordinates and may be transformed by parent nodes).
 * <br/>
 * The function creates a rectangle inside the scoping node, sets its a value,
 * reads its bounding box and then deletes the rectangle.
 *
 * @param length the length to convert
 * @param scope the scoping node
 * @return the canonical length 
 */
SVG.convertLengthToUserSpace = function( length, scope ) {
	var rect = document.createElementNS( SVG.ns, "rect" );
	rect.setAttribute( "x", length );
	rect.setAttribute( "width", 0);
	rect.setAttribute( "height", 0);
	rect.setAttribute( "opacity", 0 );
	scope.appendChild( rect );
	var length = rect.getBBox().x;
	scope.removeChild( rect );
	return length;
}

/**
 * Add a node as a sibling of an existing node.
 *
 * @param older the existing node; must have a parent
 * @param younger the new node to add after it; must be a new node
 * @return younger 
 */
SVG.addSVGSibling = function( older, younger ) {
	for( var node = older.nextSibling; node != null; node = node.nextSibling ) {
		if( node.nodeType == 1 ) {
			older.parentNode.insertBefore( younger, node );			
			return younger;
		} 
	}
	older.parentNode.appendChild( younger );
	return younger;
}
