
/**
 *  -------------------------------------
 * | JS file of generic utility methods. |
 *  -------------------------------------
 */

/**
 * Convenient method for getting object despite of how it's defined.
 */
function getHtmlElement(obj) {
    if (typeof (obj) == "string") {
        return document.getElementById(obj);
    }

    return obj;
}

/**
 * Shows/hides HTML element using CSS 'display' style.
 * Other values are used for other HTML elements.
 * If the object doesn't exist, error will be thrown.
 */
function setDisplayVisible(obj, visible) {
    obj = getHtmlElement(obj);

    if (!obj.tagName) {
        throw new Error("Object '" + obj + "' is not HTML tag.");
    }

    switch (obj.tagName) {
        case "DIV":
            obj.style.display = visible ? "block" : "none";
            break;
        case "SPAN":
            obj.style.display = visible ? "inline" : "none";
            break;
        case "TR":
            if (document.all) {
                obj.style.display = visible ? "block" : "none";
            } else { // TODO: Opera and IE7 have not been tested yet.
                obj.style.display = visible ? "table-row" : "none";
            }
            break;
        default:
            obj.style.display = visible ? "inline" : "none";
        break;
    }
}


/**
 * EventUtil object is the container for all the event-related functions.
 */
var EventUtil = new Object;
EventUtil.addEventHandler = function (oTarget, sEventType, fnHandler) {
    if (oTarget.addEventListener) {
        oTarget.addEventListener(sEventType, fnHandler, false);
    } else if (oTarget.attachEvent) {
        oTarget.attachEvent("on" + sEventType, fnHandler);
    } else {
        oTarget["on" + sEventType] = fnHandler;
    }
};

EventUtil.removeEventHandler = function (oTarget, sEventType, fnHandler) {
    if (oTarget.removeEventListener) {
        oTarget.removeEventListener(sEventType, fnHandler, false);
    } else if (oTarget.detachEvent) {
        oTarget.detachEvent("on" + sEventType, fnHandler);
    } else {
        oTarget["on" + sEventType] = null;
    }
};

EventUtil.formatEvent = function (oEvent) {
    if (isIE && isWin) {
        oEvent.charCode = (oEvent.type == "keypress") ? oEvent.keyCode : 0;
        oEvent.eventPhase = 2;
        oEvent.isChar = (oEvent.charCode > 0);
        oEvent.pageX = oEvent.clientX + document.body.scrollLeft;
        oEvent.pageY = oEvent.clientY + document.body.scrollTop;
        oEvent.preventDefault = function () {
            this.returnValue = false;
        };

        if (oEvent.type == "mouseout") {
            oEvent.relatedTarget = oEvent.toElement;
        } else if (oEvent.type == "mouseover") {
            oEvent.relatedTarget = oEvent.fromElement;
        }

        oEvent.stopPropagation = function () {
            this.cancelBubble = true;
        };

        oEvent.target = oEvent.srcElement;
        oEvent.time = (new Date).getTime();
    }
    return oEvent;
};

EventUtil.getEvent = function() {
    if (window.event) {
        return this.formatEvent(window.event);
    } else {
        return EventUtil.getEvent.caller.arguments[0];
    }
};

function toString(object, indent) {
    var ind = "    ";
    var s = "";
    if (typeof (object) == "object"
            && object.constructor == Array) {
        s += "[\n";
        //var indentNext = indent + "\t";
        var indentNext = indent + ind;
        for (var i = 0; i < object.length; ++i) {
            s += indentNext + toString(object[ i ], indentNext) + "\n";
        }
        s += indent + "]";
    } else if (typeof (object) == "object" && object.constructor == Object) {
        s += "{\n";
        var indentNext = indent + ind;
        //var indentNext = indent + "\t";
        for (var i in object) {
            s += indentNext + i + " = " + toString(object[i], indentNext) + "\n";
        }
        s += indent + "}";
    } else {
        s += object;
    }
    return s;
}

/**
 * If image.src has not been changed, browser will load it from cache. Pass random
 * parameter to force browser to reload image.
 */
function forceImageReload(image, query) {
    if(!query) {
        query = image.src;
    }
    
    var timespamp = new Date().getTime();
    
    if(query.indexOf("?") == -1) {
        query += "?" + timespamp;
    } else {
        query += "&" + timespamp;
    }
    
    image.src = query;
}

function moveFocusToBody(event) {
    if (event.keyCode == "9" ){//TAB
        getHtmlElement("body").focus();
    } 
}

function moveFocusToWorkLink(event) {
    if (event.keyCode == "27") {//ESCAPE
        window.parent.document.getElementById("processPortalWorkLink").focus();
    }
}
